﻿/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include <string.h>

#include "lpc17xx_pinsel.h"
#include "lpc17xx_timer.h"
#include "lpc17xx_uart.h"
#include "lpc17xx_ssp.h"
#include "lpc17xx_gpio.h"
#include "lpc17xx_clkpwr.h"

#include "bsp.h"
#include "SEC_AOAA.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

#define CONSOLE_DEV (LPC_UART0)

#define RX_BUF_SIZE (512)

#define RX_Q_RF      (0)
#define RX_Q_CONSOLE (1)

#define SPI_CS_ON   (bsp_pinClr(6))
#define SPI_CS_OFF  (bsp_pinSet(6))


/******************************************************************************
 * Local variables
 *****************************************************************************/

/*
 * UART receive buffer
 */

static uint8_t rxqIn[2] = {0,0};
static uint8_t rxqOut[2] = {0,0};
static uint8_t rxq[2][RX_BUF_SIZE];

static uint32_t time_ms = 0;

static bsp_comm_t commType = BSP_COMM_UART;

/******************************************************************************
 * Local functions
 *****************************************************************************/

static void rxq_put(uint8_t qIdx, uint8_t data)
{
  // full
  if (rxqOut[qIdx] == (rxqIn[qIdx] + 1) % RX_BUF_SIZE) {
    return;
  }
  rxq[qIdx][rxqIn[qIdx]] = data;
  rxqIn[qIdx] = (rxqIn[qIdx] + 1) % RX_BUF_SIZE;
}

static uint8_t rxq_get(uint8_t qIdx)
{
  uint8_t d = 0;
  // empty
  if (rxqIn[qIdx] == rxqOut[qIdx]) {
    return 0;
  }

  d = rxq[qIdx][rxqOut[qIdx]];
  rxqOut[qIdx] = (rxqOut[qIdx] + 1) % RX_BUF_SIZE;

  return d;
}

static uint8_t rxq_isEmpty(uint8_t qIdx)
{
  return (rxqIn[qIdx] == rxqOut[qIdx]);
}

static void rf_uartRecvCb(void)
{
  uint8_t data = 0;
  uint32_t len = 0;

  while(1) {
      len = UART_Receive(SEC14_UART_DEV, &data, 1, NONE_BLOCKING);

      if (len) {
        rxq_put(RX_Q_RF, data);
      }

      // there is no more data
      else {

        break;
      }
  }
}

static void console_uartRecvCb(void)
{
  uint8_t data = 0;
  uint32_t len = 0;

  while(1) {
      len = UART_Receive(CONSOLE_DEV, &data, 1, NONE_BLOCKING);

      if (len) {
        rxq_put(RX_Q_CONSOLE, data);
      }

      // there is no more data
      else {

        break;
      }
  }
}

/******************************************************************************
 * Public functions
 *****************************************************************************/

/******************************************************************************
 *
 * Description:
 *   BSP initialize function. Must be called before any other BSP related
 *   functions.
 *
 *****************************************************************************/
void bsp_init(void)
{
  UART_CFG_Type uartCfg;
  PINSEL_CFG_Type PinCfg;

  // =======================
  // Timer related initialization
  // =======================

  // 10 ms sys tick
  SysTick_Config(SystemCoreClock/100);



  // =======================
  // initialize UART connected for console output
  // =======================

  // p0.1 and p0.2 as UART0
  PinCfg.Funcnum = 1;
  PinCfg.Portnum = 0;
  PinCfg.Pinnum = 2;
  PINSEL_ConfigPin(&PinCfg);
  PinCfg.Pinnum = 3;
  PINSEL_ConfigPin(&PinCfg);

  uartCfg.Baud_rate = 115200;
  uartCfg.Databits = UART_DATABIT_8;
  uartCfg.Parity = UART_PARITY_NONE;
  uartCfg.Stopbits = UART_STOPBIT_1;

  UART_Init(CONSOLE_DEV, &uartCfg);
  UART_TxCmd(CONSOLE_DEV, ENABLE);

  NVIC_EnableIRQ(UART0_IRQn);

  // =======================
  // Button and LED
  // =======================

  // SW2 button
  GPIO_SetDir( 2, (1<<11), 0 );

  // RGB LED6
  GPIO_SetDir( 2, (1<<0), 1 );
  GPIO_SetDir( 2, (1<<1), 1 );
  GPIO_SetDir( 2, (1<<2), 1 );
  // turn leds off
  GPIO_SetValue( 2, (1<<0|1<<1|1<<2));
}

/******************************************************************************
 *
 * Description:
 *   Initialize the RF interface. Must be called before any other RF
 *   related function is called.
 *
 *****************************************************************************/
void bsp_rf_init(bsp_comm_t type, bsp_comm_arg_t* args)
{
  UART_CFG_Type uartCfg;
  PINSEL_CFG_Type pinCfg;
  SSP_CFG_Type sspCfg;

  commType = type;

  if (type == BSP_COMM_UART) {

    // =======================
    // initialize UART connected to Serial Expansion Connector
    // =======================

    SEC14_INIT_UART;

    uartCfg.Baud_rate = args->arg1;;
    uartCfg.Databits = (UART_DATABIT_Type)args->arg2;
    uartCfg.Stopbits = (UART_STOPBIT_Type)args->arg3;
    uartCfg.Parity   = (UART_PARITY_Type)args->arg4;

    UART_Init(SEC14_UART_DEV, &uartCfg);

    UART_TxCmd(SEC14_UART_DEV, ENABLE);
    UART_IntConfig(SEC14_UART_DEV, UART_INTCFG_RBR, ENABLE);

    // UART3 is connected to Serial Expansion Connector
    NVIC_EnableIRQ(UART3_IRQn);

  } else if (type == BSP_COMM_SPI) {

    pinCfg.Funcnum = 2;
    pinCfg.Portnum = 0;
    pinCfg.Pinnum = 7;
    PINSEL_ConfigPin(&pinCfg);
    pinCfg.Pinnum = 8;
    PINSEL_ConfigPin(&pinCfg);
    pinCfg.Pinnum = 9;
    PINSEL_ConfigPin(&pinCfg);

    // set SEC pin 6 to output for SSEL
    bsp_pinDir(6, 1);

    // Initialize SSP0
    SSP_ConfigStructInit(&sspCfg);
    sspCfg.ClockRate = args->arg1;
    if (args->arg2 == BSP_ARG2_SPI_CPHA_FIRST) {
      sspCfg.CPHA = SSP_CPHA_FIRST;
    }
    else {
      sspCfg.CPHA = SSP_CPHA_SECOND;
    }

    if (args->arg3 == BSP_ARG3_SPI_CPOL_HI) {
      sspCfg.CPOL = SSP_CPOL_HI;
    }
    else {
      sspCfg.CPOL = SSP_CPOL_LO;
    }

    switch (args->arg4) {
    case BSP_ARG4_SPI_DATABIT_8:
      sspCfg.Databit = SSP_DATABIT_8;
      break;
    }


    SSP_Init(SEC14_SPI_DEV, &sspCfg);
    SSP_Cmd(SEC14_SPI_DEV, ENABLE);

    SPI_CS_OFF;


  }
}

/******************************************************************************
 *
 * Description:
 *   Set direction of a PIN
 *
 * Params:
 *   [in] pin - PIN number according to SEC numbering (1-14)
 *   [in] dir - set 1 for output and 0 for input
 *
 *****************************************************************************/
void bsp_pinDir(uint8_t pin, uint8_t dir)
{
  switch(pin) {

  case 6:
    // p0.6
    GPIO_SetDir(0, (1<<6), dir);

    break;

  case 7:
    // p4.29
    GPIO_SetDir(4, (1<<29), dir);

    break;
  case 8:
    // p4.28
    GPIO_SetDir(4, (1<<28), dir);

    break;
  case 11:
    // p3.25
    GPIO_SetDir(3, (1<<25), dir);

    break;
  case 12:
    // p3.26
    GPIO_SetDir(3, (1<<26), dir);

    break;
  case 13:
    // p0.23
    GPIO_SetDir(0, (1<<23), dir);

    break;
  case 14:
    // p0.26
    GPIO_SetDir(0, (1<<26), dir);

    break;
  default:
    // ignore other pin values
    break;
  }
}

/******************************************************************************
 *
 * Description:
 *   Set PIN output high
 *
 * Params:
 *   [in] pin - PIN number according to SEC numbering (1-14)
 *
 *****************************************************************************/
void bsp_pinSet(uint8_t pin)
{
  switch(pin) {

  case 6:
    GPIO_SetValue(0, (1<<6));
    break;

  case 7:
    GPIO_SetValue(4, (1<<29));
    break;
  case 8:
    GPIO_SetValue(4, (1<<28));
    break;
  case 11:
    GPIO_SetValue(3, (1<<25));
    break;
  case 12:
    GPIO_SetValue(3, (1<<26));
    break;
  case 13:
    GPIO_SetValue(0, (1<<23));
    break;
  case 14:
    GPIO_SetValue(0, (1<<26));
    break;
  default:
    // ignore other pin values
    break;
  }
}

/******************************************************************************
 *
 * Description:
 *   Set PIN output low
 *
 * Params:
 *   [in] pin - PIN number according to SEC numbering (1-14)
 *
 *****************************************************************************/
void bsp_pinClr(uint8_t pin)
{
  switch(pin) {

  case 6:
    GPIO_ClearValue(0, (1<<6));
    break;

  case 7:
    GPIO_ClearValue(4, (1<<29));
    break;
  case 8:
    GPIO_ClearValue(4, (1<<28));
    break;
  case 11:
    GPIO_ClearValue(3, (1<<25));
    break;
  case 12:
    GPIO_ClearValue(3, (1<<26));
    break;
  case 13:
    GPIO_ClearValue(0, (1<<23));
    break;
  case 14:
    GPIO_ClearValue(0, (1<<26));
    break;
  default:
    // ignore other pin values
    break;
  }
}

/******************************************************************************
 *
 * Description:
 *   Send data to the RF module.
 *
 * Params:
 *   [in] txbuf - buffer containing data to send
 *   [in] buflen - number of bytes to send
 *   [in] blocking - 1=blocking transfer, 0= non-blocking
 *
 * Returns:
 *   Number of bytes sent.
 *
 *****************************************************************************/
uint32_t bsp_rf_send(uint8_t *txbuf, uint32_t buflen,
    uint8_t blocking)
{
  TRANSFER_BLOCK_Type flag = BLOCKING;

  if (commType != BSP_COMM_UART) {
    return 0;
  }

  if (!blocking) {
    flag = NONE_BLOCKING;
  }
  return UART_Send(SEC14_UART_DEV, txbuf, buflen, flag);
}

/******************************************************************************
 *
 * Description:
 *   Send a null-terminates string to the RF module.
 *
 * Params:
 *   [in] str - the string to send
 *
 * Returns:
 *   Number of bytes sent.
 *
 *****************************************************************************/
uint32_t bsp_rf_sendString(uint8_t *str)
{
  if (commType != BSP_COMM_UART) {
    return 0;
  }

  return UART_Send(SEC14_UART_DEV, str, strlen((char*)str), BLOCKING);
}

/******************************************************************************
 *
 * Description:
 *   Receive a block of data from the RF module.
 *
 * Params:
 *   [in] rxbuf - pointer to receive buffer
 *   [in] buflen - length of buffer
 *
 * Returns:
 *   Number of bytes sent.
 *
 *****************************************************************************/
uint32_t bsp_rf_receive(uint8_t *buf, uint32_t buflen)
{

  uint32_t pos = 0;

  if (commType != BSP_COMM_UART) {
    return 0;
  }

  while(buflen > 0 && !rxq_isEmpty(RX_Q_RF)) {
    buf[pos] = rxq_get(RX_Q_RF);
    pos++;
    buflen--;
  }

  return pos;
}

/******************************************************************************
 *
 * Description:
 *   Check if the receive queue is empty
 *
 *****************************************************************************/
uint8_t bsp_rf_recvIsEmpty(void)
{
  if (commType != BSP_COMM_UART) {
    return 1;
  }

  return rxq_isEmpty(RX_Q_RF);
}

/******************************************************************************
 *
 * Description:
 *   Clear receive buffer
 *
 *****************************************************************************/
void bsp_rf_clearRx(void)
{

  if (commType != BSP_COMM_UART) {
    return;
  }

  NVIC_DisableIRQ(UART3_IRQn);

  rxqIn[RX_Q_RF] = 0;
  rxqOut[RX_Q_RF] = 0;

  NVIC_EnableIRQ(UART3_IRQn);
}

/******************************************************************************
 *
 * Description:
 *   Communicate with the RF module through SPI/SSP interface.
 *
 * Params:
 *   [in] wrBuf - buffer containing data to write
 *   [in] wrLen - number of bytes to write
 *   [in] rdBuf - buffer for data to read
 *   [in] rdLen - number of bytes to read
 *
 * Returns:
 *   Number of bytes written or read.
 *
 *****************************************************************************/
uint32_t bsp_rf_spiWriteRead(uint8_t* wrBuf, uint32_t wrLen,
    uint8_t* rdBuf, uint32_t rdLen)
{
  uint32_t txLen = 0;
  SSP_DATA_SETUP_Type sspCfg;

  if (commType == BSP_COMM_UART) {
    return 0;
  }

  SPI_CS_ON;

  if (wrBuf != NULL && wrLen > 0) {
    sspCfg.length = wrLen;
    sspCfg.tx_data = wrBuf;
    sspCfg.rx_data = NULL;

    txLen = SSP_ReadWrite(SEC14_SPI_DEV, &sspCfg, SSP_TRANSFER_POLLING);
  }

  if (rdBuf != NULL && rdLen > 0) {
    sspCfg.length = rdLen;
    sspCfg.tx_data = NULL;
    sspCfg.rx_data = rdBuf;

    txLen = SSP_ReadWrite(SEC14_SPI_DEV, &sspCfg, SSP_TRANSFER_POLLING);
  }


  SPI_CS_OFF;

  return txLen;
}

/******************************************************************************
 *
 * Description:
 *   Send data to the console.
 *
 * Params:
 *   [in] txbuf - buffer containing data to send
 *   [in] buflen - number of bytes to send
 *   [in] blocking - 1=blocking transfer, 0= non-blocking
 *
 * Returns:
 *   Number of bytes sent.
 *
 *****************************************************************************/
uint32_t bsp_console_send(uint8_t *txbuf, uint32_t buflen,
    uint8_t blocking)
{
  TRANSFER_BLOCK_Type flag = BLOCKING;
  if (!blocking) {
    flag = NONE_BLOCKING;
  }
  return UART_Send(CONSOLE_DEV, txbuf, buflen, flag);
}

/******************************************************************************
 *
 * Description:
 *   Send a null-terminates string to the console.
 *
 * Params:
 *   [in] str - the string to send
 *
 * Returns:
 *   Number of bytes sent.
 *
 *****************************************************************************/
uint32_t bsp_console_sendString(uint8_t *str)
{
  return UART_Send(CONSOLE_DEV, str, strlen((char*)str), BLOCKING);
}

/******************************************************************************
 *
 * Description:
 *   Receive a block of data from the console.
 *
 * Params:
 *   [in] rxbuf - pointer to receive buffer
 *   [in] buflen - length of buffer
 *
 * Returns:
 *   Number of bytes sent.
 *
 *****************************************************************************/
uint32_t bsp_console_receive(uint8_t *buf, uint32_t buflen)
{

  uint32_t pos = 0;

  while(buflen > 0 && !rxq_isEmpty(RX_Q_CONSOLE)) {
    buf[pos] = rxq_get(RX_Q_CONSOLE);
    pos++;
    buflen--;
  }

  return pos;
}

/******************************************************************************
 *
 * Description:
 *   Delay specified number of milliseconds
 *
 * Params:
 *   [in] ms - number of milliseconds to delay
 *
 *****************************************************************************/
void bsp_delayMs(uint32_t ms)
{
  Timer0_Wait(ms);
}

/******************************************************************************
 *
 * Description:
 *   Delay specified number of microseconds
 *
 * Params:
 *   [in] us - number of microseconds to delay
 *
 *****************************************************************************/
void bsp_delayUs(uint32_t us)
{
  Timer0_us_Wait(us);
}

/******************************************************************************
 *
 * Description:
 *   Delay specified number of microseconds
 *
 * Params:
 *   [in] us - number of microseconds to delay
 *
 *****************************************************************************/
uint32_t bsp_timeGet(void)
{
  return time_ms;
}

/******************************************************************************
 *
 * Description:
 *   Get button state
 *
 * Returns:
 *   1 if the button is pressed; otherwise 0
 *
 *****************************************************************************/
uint8_t bsp_getButtonState(void)
{
  uint32_t v = GPIO_ReadValue(2);

  //
  // SW2 button on the AOAA board
  //

  if ((v & (1<<11)) == 0) {
    return 1;
  }
  else {
    return 0;
  }
}

/******************************************************************************
 *
 * Description:
 *   Turn LED on or off
 *
 * Params:
 *   [in] on - 1 to turn on; 0 to turn off
 *
 *****************************************************************************/
void bsp_setLed(uint8_t on)
{

  //
  // LED6 on the AOAA board
  //

  if (on) {
    // turning on the RED LED
    GPIO_ClearValue( 2, (1<<0) );
  }
  else {
    GPIO_SetValue( 2, (1<<0));
  }

}

/******************************************************************************
 *
 * Description:
 *   UART3 interrupt handler
 *
 *****************************************************************************/
void UART3_IRQHandler(void)
{
  uint32_t intsrc;


  // interrupt source
  intsrc = LPC_UART3->IIR & UART_IIR_INTID_MASK;

  if (intsrc == UART_IIR_INTID_RDA || intsrc == UART_IIR_INTID_CTI) {
    rf_uartRecvCb();
  }
}

/******************************************************************************
 *
 * Description:
 *   UART3 interrupt handler
 *
 *****************************************************************************/
void UART0_IRQHandler(void)
{
  uint32_t intsrc;


  // interrupt source
  intsrc = LPC_UART0->IIR & UART_IIR_INTID_MASK;

  if (intsrc == UART_IIR_INTID_RDA || intsrc == UART_IIR_INTID_CTI) {
    console_uartRecvCb();
  }
}

/******************************************************************************
 *
 * Description:
 *   Systick Handler
 *
 *****************************************************************************/
void SysTick_Handler(void)
{
  time_ms += 10;
}

