/*****************************************************************************
 *
 *
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/*
 * Implementation based on Alvaro Prieto's c2500 project
 *
 * https://github.com/alvarop/msp430-cc2500
 */

/******************************************************************************
 * Includes
 *****************************************************************************/
#include <string.h>
#include <stdio.h>

#include "lpc_types.h"

#include "eadebug.h"
#include "bsp.h"
#include "error.h"



/******************************************************************************
 * Typdefs and defines
 *****************************************************************************/

#define CC2500_READ_SINGLE (0x80)
#define CC2500_READ_BURST  (0xC0)
#define CC2500_WRITE_BURST (0x40)

// configuration registers
#define CC2500_REG_IOCFG2   0x00 // GDO2 output pin configuration
#define CC2500_REG_IOCFG1   0x01 // GDO1 output pin configuration
#define CC2500_REG_IOCFG0   0x02 // GDO0 output pin configuration
#define CC2500_REG_FIFOTHR  0x03 // RX FIFO and TX FIFO thresholds
#define CC2500_REG_SYNC1    0x04 // Sync word, high byte
#define CC2500_REG_SYNC0    0x05 // Sync word, low byte
#define CC2500_REG_PKTLEN   0x06 // Packet length
#define CC2500_REG_PKTCTRL1 0x07 // Packet automation control
#define CC2500_REG_PKTCTRL0 0x08 // Packet automation control
#define CC2500_REG_ADDR     0x09 // Device address
#define CC2500_REG_CHANNR   0x0A // Channel number
#define CC2500_REG_FSCTRL1  0x0B // Frequency synthesizer control
#define CC2500_REG_FSCTRL0  0x0C // Frequency synthesizer control
#define CC2500_REG_FREQ2    0x0D // Frequency control word, high byte
#define CC2500_REG_FREQ1    0x0E // Frequency control word, middle byte
#define CC2500_REG_FREQ0    0x0F // Frequency control word, low byte
#define CC2500_REG_MDMCFG4  0x10 // Modem configuration
#define CC2500_REG_MDMCFG3  0x11 // Modem configuration
#define CC2500_REG_MDMCFG2  0x12 // Modem configuration
#define CC2500_REG_MDMCFG1  0x13 // Modem configuration
#define CC2500_REG_MDMCFG0  0x14 // Modem configuration
#define CC2500_REG_DEVIATN  0x15 // Modem deviation setting
#define CC2500_REG_MCSM2    0x16 // Main Radio Cntrl State Machine config
#define CC2500_REG_MCSM1    0x17 // Main Radio Cntrl State Machine config
#define CC2500_REG_MCSM0    0x18 // Main Radio Cntrl State Machine config
#define CC2500_REG_FOCCFG   0x19 // Frequency Offset Compensation config
#define CC2500_REG_BSCFG    0x1A // Bit Synchronization configuration
#define CC2500_REG_AGCCTRL2 0x1B // AGC control
#define CC2500_REG_AGCCTRL1 0x1C // AGC control
#define CC2500_REG_AGCCTRL0 0x1D // AGC control
#define CC2500_REG_WOREVT1  0x1E // High byte Event 0 timeout
#define CC2500_REG_WOREVT0  0x1F // Low byte Event 0 timeout
#define CC2500_REG_WORCTRL  0x20 // Wake On Radio control
#define CC2500_REG_FREND1   0x21 // Front end RX configuration
#define CC2500_REG_FREND0   0x22 // Front end TX configuration
#define CC2500_REG_FSCAL3   0x23 // Frequency synthesizer calibration
#define CC2500_REG_FSCAL2   0x24 // Frequency synthesizer calibration
#define CC2500_REG_FSCAL1   0x25 // Frequency synthesizer calibration
#define CC2500_REG_FSCAL0   0x26 // Frequency synthesizer calibration
#define CC2500_REG_RCCTRL1  0x27 // RC oscillator configuration
#define CC2500_REG_RCCTRL0  0x28 // RC oscillator configuration
#define CC2500_REG_FSTEST   0x29 // Frequency synthesizer cal control
#define CC2500_REG_PTEST    0x2A // Production test
#define CC2500_REG_AGCTEST  0x2B // AGC test
#define CC2500_REG_TEST2    0x2C // Various test settings
#define CC2500_REG_TEST1    0x2D // Various test settings
#define CC2500_REG_TEST0    0x2E // Various test settings

// Status registers
#define CC2500_REG_PARTNUM     0x30 // Part number
#define CC2500_REG_VERSION     0x31 // Current version number
#define CC2500_REG_FREQEST     0x32 // Frequency offset estimate
#define CC2500_REG_LQI         0x33 // Demodulator estimate for link quality
#define CC2500_REG_RSSI        0x34 // Received signal strength indication
#define CC2500_REG_MARCSTATE   0x35 // Control state machine state
#define CC2500_REG_WORTIME1    0x36 // High byte of WOR timer
#define CC2500_REG_WORTIME0    0x37 // Low byte of WOR timer
#define CC2500_REG_PKTSTATUS   0x38 // Current GDOx status and packet status
#define CC2500_REG_VCO_VC_DAC  0x39 // Current setting from PLL cal module
#define CC2500_REG_TXBYTES     0x3A // Underflow and # of bytes in TXFIFO
#define CC2500_REG_RXBYTES     0x3B // Overflow and # of bytes in RXFIFO
#define CC2500_REG_NUM_RXBYTES 0x7F // Mask "# of bytes" field in _RXBYTES

#define CC2500_DEF_PARTNUM (0x80)
#define CC2500_DEF_VERSION (0x03)

// Strobe commands
#define CC2500_REG_SRES    0x30 // Reset chip.
#define CC2500_REG_SFSTXON 0x31 // Enable/calibrate freq synthesizer
#define CC2500_REG_SXOFF   0x32 // Turn off crystal oscillator.
#define CC2500_REG_SCAL    0x33 // Calibrate freq synthesizer & disable
#define CC2500_REG_SRX     0x34 // Enable RX.
#define CC2500_REG_STX     0x35 // Enable TX.
#define CC2500_REG_SIDLE   0x36 // Exit RX / TX
#define CC2500_REG_SAFC    0x37 // AFC adjustment of freq synthesizer
#define CC2500_REG_SWOR    0x38 // Start automatic RX polling sequence
#define CC2500_REG_SPWD    0x39 // Enter pwr down mode when CSn goes hi
#define CC2500_REG_SFRX    0x3A // Flush the RX FIFO buffer.
#define CC2500_REG_SFTX    0x3B // Flush the TX FIFO buffer.
#define CC2500_REG_SWORRST 0x3C // Reset real time clock.
#define CC2500_REG_SNOP    0x3D // No operation.


#define CC2500_PATABLE (0x3E)
#define CC2500_TXFIFO  (0x3F)
#define CC2500_RXFIFO  (0x3F)

#define CC2500_BUF_SZ (64)

#define CC2500_BUF_IDX_REG  (0)
#define CC2500_BUF_IDX_LEN  (1)
#define CC2500_BUF_IDX_DST  (2)
#define CC2500_BUF_IDX_DATA (3)

/******************************************************************************
 * Local variables
 *****************************************************************************/

static uint8_t rfState = 0;
static char* stateStr[] = {
    "SLEEP",
    "IDLE",
    "XOFF",
    "VCOON_MC",
    "REGON_MC",
    "MANCAL",
    "VCOON",
    "REGON",
    "STARTCAL",
    "BWBOOST",
    "FS_LOCK",
    "IFADCON",
    "ENDCAL",
    "RX",
    "RX_END",
    "RX_RST",
    "TXRX_SWITCH",
    "RXFIFO_OVERFLOW",
    "FSTXON",
    "TX",
    "TX_END",
    "RXTX_SWITCH",
    "TXFIFO_UNDERFLOW"
};
static uint8_t txBuffer[CC2500_BUF_SZ+1];


/******************************************************************************
 * Local functions
 *****************************************************************************/

/******************************************************************************
 *
 * Description:
 *   Read a register.
 *
 *****************************************************************************/
static uint8_t readRegister(uint8_t reg)
{
  uint8_t data;

  reg = (reg | CC2500_READ_SINGLE);

  bsp_rf_spiWriteRead(&reg, 1, &data, 1);

  return data;
}

/******************************************************************************
 *
 * Description:
 *   Write to a register.
 *
 *****************************************************************************/
static void writeRegister(uint8_t reg, uint8_t val)
{
  uint8_t data[2];
  data[0] = reg;
  data[1] = val;
  bsp_rf_spiWriteRead(data, 2, NULL, 0);
}

/******************************************************************************
 *
 * Description:
 *   Send strobe command.
 *
 *****************************************************************************/
static void strobe(uint8_t reg)
{
  bsp_rf_spiWriteRead(&reg, 1, NULL, 0);
}

/******************************************************************************
 *
 * Description:
 *   Read status register.
 *
 *****************************************************************************/
static uint8_t readStatus(uint8_t reg)
{
  uint8_t data;

  reg = (reg | CC2500_READ_BURST);

  bsp_rf_spiWriteRead(&reg, 1, &data, 1);

  return data;
}

/******************************************************************************
 *
 * Description:
 *   Burst read.
 *
 *****************************************************************************/
static void readBurst(uint8_t reg, uint8_t* buf, uint8_t len)
{
  reg = (reg | CC2500_READ_BURST);
  bsp_rf_spiWriteRead(&reg, 1, buf, len);
}

/******************************************************************************
 *
 * Description:
 *   Burst transfer.
 *
 *****************************************************************************/
static void writeBurst(uint8_t* buf, uint8_t len)
{
  // register
  buf[CC2500_BUF_IDX_REG] |= CC2500_WRITE_BURST;

  bsp_rf_spiWriteRead(buf, len, NULL, 0);
}

/******************************************************************************
 *
 * Description:
 *   Configure the RF module.
 *
 *****************************************************************************/
static void rfConfig(void)
{

  writeRegister(CC2500_REG_IOCFG2, 0x0E); // GDO2 output pin config.
  writeRegister(CC2500_REG_IOCFG0, 0x06); // GDO0 output pin config.
  writeRegister(CC2500_REG_PKTLEN, 0x3D); // Packet length.
  writeRegister(CC2500_REG_PKTCTRL1, 0x0E); // Packet automation control.
  writeRegister(CC2500_REG_PKTCTRL0, 0x05); // Packet automation control.
  writeRegister(CC2500_REG_ADDR, 0x01); // Device address.
  writeRegister(CC2500_REG_CHANNR, 0x00); // Channel number.
  writeRegister(CC2500_REG_FSCTRL1, 0x07); // Freq synthesizer control.
  writeRegister(CC2500_REG_FSCTRL0, 0x00); // Freq synthesizer control.
  writeRegister(CC2500_REG_FREQ2, 0x5D); // Freq control word, high byte
  writeRegister(CC2500_REG_FREQ1, 0x93); // Freq control word, mid byte.
  writeRegister(CC2500_REG_FREQ0, 0xB1); // Freq control word, low byte.
  writeRegister(CC2500_REG_MDMCFG4, 0x2D); // Modem configuration.
  writeRegister(CC2500_REG_MDMCFG3, 0x3B); // Modem configuration.
  writeRegister(CC2500_REG_MDMCFG2, 0x73); // Modem configuration.
  writeRegister(CC2500_REG_MDMCFG1, 0x22); // Modem configuration.
  writeRegister(CC2500_REG_MDMCFG0, 0xF8); // Modem configuration.
  writeRegister(CC2500_REG_DEVIATN, 0x00); // Modem dev (when FSK mod en)
  writeRegister(CC2500_REG_MCSM1 , 0x2F); //MainRadio Cntrl State Machine
  writeRegister(CC2500_REG_MCSM0 , 0x18); //MainRadio Cntrl State Machine
  writeRegister(CC2500_REG_FOCCFG, 0x1D); // Freq Offset Compens. Config
  writeRegister(CC2500_REG_BSCFG, 0x1C); // Bit synchronization config.
  writeRegister(CC2500_REG_AGCCTRL2, 0xC7); // AGC control.
  writeRegister(CC2500_REG_AGCCTRL1, 0x00); // AGC control.
  writeRegister(CC2500_REG_AGCCTRL0, 0xB2); // AGC control.
  writeRegister(CC2500_REG_FREND1, 0xB6); // Front end RX configuration.
  writeRegister(CC2500_REG_FREND0, 0x10); // Front end RX configuration.
  writeRegister(CC2500_REG_FSCAL3, 0xEA); // Frequency synthesizer cal.
  writeRegister(CC2500_REG_FSCAL2, 0x0A); // Frequency synthesizer cal.
  writeRegister(CC2500_REG_FSCAL1, 0x00); // Frequency synthesizer cal.
  writeRegister(CC2500_REG_FSCAL0, 0x11); // Frequency synthesizer cal.
  writeRegister(CC2500_REG_FSTEST, 0x59); // Frequency synthesizer cal.
  writeRegister(CC2500_REG_TEST2, 0x88); // Various test settings.
  writeRegister(CC2500_REG_TEST1, 0x31); // Various test settings.
  writeRegister(CC2500_REG_TEST0, 0x0B); // Various test settings.

}

/******************************************************************************
 * Public functions
 *****************************************************************************/

/******************************************************************************
 *
 * Description:
 *   Initialize the RF module.
 *
 *****************************************************************************/
error_t an2500_init(void)
{
  uint8_t partNum;
  uint8_t version;
  uint8_t data[2];
  uint8_t marcState = 0;

  bsp_comm_arg_t args;

  args.arg1 = 1000000;
  args.arg2 = BSP_ARG2_SPI_CPHA_FIRST;
  args.arg3 = BSP_ARG3_SPI_CPOL_HI;
  args.arg4 = BSP_ARG4_SPI_DATABIT_8;


  bsp_rf_init(BSP_COMM_SPI, &args);

  // reset
  data[0] = CC2500_REG_SRES;
  bsp_rf_spiWriteRead(data, 1, NULL, 0);

  bsp_delayMs(100);

  partNum = readStatus(CC2500_REG_PARTNUM);
  version = readStatus(CC2500_REG_VERSION);
  if (partNum != CC2500_DEF_PARTNUM && version != CC2500_DEF_VERSION) {
    dbg("an2500: Unknown part: %x (%x), version: %x (%x)\r\n",
        partNum, CC2500_DEF_PARTNUM, version, CC2500_DEF_VERSION);
    return ERR_NOT_INIT;
  }

  marcState = readStatus(CC2500_REG_MARCSTATE);

  rfConfig();

  // power
  data[0] = CC2500_PATABLE;
  data[1] = 0xFB;
  writeBurst(data, 2);

  // change to RX mode
  strobe(CC2500_REG_SRX);

  marcState = readStatus(CC2500_REG_MARCSTATE);

  return ERR_OK;
}

/******************************************************************************
 *
 * Description:
 *   Send data via the RF module to a destination module
 *
 * Params:
 *   [in] buf - buffer with data to send
 *   [in] len - size of the buffer
 *   [in] destination - destination address
 *
 * Returns:
 *   number of written bytes
 *
 *****************************************************************************/
uint8_t an2500_send(uint8_t* buf, uint8_t len, uint8_t destination)
{
  if (len + 2 > CC2500_BUF_SZ) {
    return 0;
  }


  txBuffer[CC2500_BUF_IDX_REG] = CC2500_TXFIFO;
  // length field
  txBuffer[CC2500_BUF_IDX_LEN] = len + 1;
  // destination field
  txBuffer[CC2500_BUF_IDX_DST] = destination;

  memcpy(&txBuffer[CC2500_BUF_IDX_DATA], buf, len);

  writeBurst(txBuffer, len + 3);
  // change state to TX
  strobe(CC2500_REG_STX);


  return len;
}


/******************************************************************************
 *
 * Description:
 *   Receive data from the RF module.
 *
 * Params:
 *   [in] buf - read data is stored in this buffer
 *   [in] len - size of the buffer
 *
 * Returns:
 *   number of read bytes
 *
 *****************************************************************************/
uint8_t an2500_receive(uint8_t* buf, uint8_t len)
{
  uint8_t numBytes;
  uint8_t status[2];
  uint8_t marcState;

  // output state changes in the console
  marcState = (readStatus(CC2500_REG_MARCSTATE) & 0x1f);
  if (marcState != rfState) {
    rfState = marcState;
    dbg("state=%s\r\n", stateStr[rfState]);
  }

  numBytes = readStatus(CC2500_REG_RXBYTES);

  // overflow
  if (numBytes & 0x80) {
    dbg("an2500: RXFIFO overflow. Flushing\r\n");
    strobe(CC2500_REG_SFRX);
    return 0;
  }

  numBytes = (numBytes & CC2500_REG_NUM_RXBYTES);

  if (numBytes > 0) {

    // read first byte from FIFO (contains packet length)
    numBytes = readRegister(CC2500_RXFIFO);

    if (numBytes <= len) {
      dbg("an2500: reading %d bytes\r\n", numBytes);
      readBurst(CC2500_RXFIFO, buf, numBytes);

      // read LQI and CRC
      readBurst(CC2500_RXFIFO, status, 2);
      if ((status[1] & 0x80) == 0) {
        dbg("an2500: Bad CRC RSSI=%x, LQI=%x\r\n", status[0], status[1]);
        numBytes = 0;
      }

    }
    else {
      dbg("an2500: Buffer too small (%d < %d). Flushing\r\n", len, numBytes);
      numBytes = 0;
      strobe(CC2500_REG_SFRX);
    }

  }


  return numBytes;
}

