/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include <string.h>
#include <stdio.h>

#include "lpc_types.h"

#include "eadebug.h"
#include "bsp.h"
#include "rnxv.h"

/******************************************************************************
 * Typdefs and defines
 *****************************************************************************/

#define RNXV_END_CMD ("\r")
#define CR (0x0D)
#define LF (0x0A)
#define RESP_OK ("AOK")
#define CMD_MODE_OK ("CMD")

#define ERROR_BUF_TOO_SMALL (-1)
#define ERROR_TIMEOUT       (-2)

/******************************************************************************
 * Local variables
 *****************************************************************************/


/******************************************************************************
 * Local functions
 *****************************************************************************/


/******************************************************************************
 *
 * Description:
 *   Read data from UART up until CR is found.
 *
 * Params:
 *   [in] buf - read data is copied to this buffer
 *   [in] bufLen - size of buffer
 *   [in] timeout - max number of ms to wait for data
 *
 * Returns:
 *   number of read bytes or a negative number if an error occurs
 *
 *****************************************************************************/
static int32_t readLine(uint8_t* buf, uint32_t bufLen, uint32_t timeout)
{
  uint32_t pos = 0;
  uint32_t len = 0;
  uint32_t t = 0;
  uint8_t data;
  uint8_t startFound = 0;

  t = bsp_timeGet() + timeout;


  while(pos < bufLen && t > bsp_timeGet()) {

    len = bsp_rf_receive(&data, 1);

    if (len > 0) {

      // wait until non CR LF is found
      if (!startFound) {
        if (data == LF || data == CR) {
          continue;
        }

        startFound = 1;
      }

      buf[pos] = data;

      if (buf[pos] == LF) {
        buf[pos] = '\0';
        if (pos > 1 && buf[pos-1] == CR) {
          buf[pos-1] = '\0';
          pos--;
        }
        break;
      }

      pos += len;

    }

  }

  if (pos >= bufLen) {
    return ERROR_BUF_TOO_SMALL;
  }

  if (t <= bsp_timeGet()) {
    return ERROR_TIMEOUT;
  }

  return pos;
}

/******************************************************************************
 *
 * Description:
 *   Set the XBee module in command mode
 *
 * Returns:
 *   ERR_OK or an error code if the command failed
 *
 *****************************************************************************/
static error_t commandMode(void)
{
  uint8_t ebuf[100];
  int32_t lineLen = 0;
  error_t err = ERR_RF_CMD_ERROR;
  int i = 0;

  // try at most 10 times to go into command mode
  for (i = 0; i < 10; i++) {

    bsp_rf_clearRx();
    bsp_rf_sendString((uint8_t*)"$$$");
    bsp_delayMs(100);

    lineLen = readLine(ebuf, 100, 1200);

    if (lineLen > 0/* && strcmp(CMD_MODE_OK, (char*)ebuf) != 0*/) {

      if (strcmp(CMD_MODE_OK, (char*)ebuf) == 0) {
        err = ERR_OK;
        break;
      }

    }

  }

  return err;
}

/******************************************************************************
 *
 * Description:
 *   AT command that expects a single line response
 *
 * Params:
 *   [in] atCmd - the AT command string
 *   [in] resp - response buffer
 *   [in] respLen - size of response buffer
 *
 * Returns:
 *   ERR_OK or an error code if the command failed
 *
 *****************************************************************************/
static error_t cmdGet(uint8_t* cmd, uint8_t* resp, uint32_t respLen)
{

  int32_t lineLen = 0;
  error_t err = ERR_OK;

  bsp_rf_clearRx();
  bsp_rf_sendString(cmd);
  bsp_rf_sendString((uint8_t*)RNXV_END_CMD);

  bsp_delayMs(450);

  do {

    // a response is expected
    if (resp != NULL && respLen > 0) {
      lineLen = readLine(resp, respLen, 1000);

      if (lineLen < 0) {
        // error while reading
        err = ERR_RF_READ_ERROR;
        break;
      }

      // check if the command is echoed
      if (strncmp((char*)cmd, (char*)resp, strlen((char*)cmd)) == 0) {
        // skip the echoed command and read one more line

        lineLen = readLine(resp, respLen, 1000);

        if (lineLen < 0) {
          // error while reading
          err = ERR_RF_READ_ERROR;
          break;
        }


      }

    }

  } while(0);

  return err;
}

/******************************************************************************
 *
 * Description:
 *   AT command that sets a value and only expects OK or ERROR as result
 *
 * Params:
 *   [in] atCmd - the AT command string
 *
 * Returns:
 *   ERR_OK or an error code if the command failed
 *
 *****************************************************************************/
static error_t cmdSet(uint8_t*cmd)
{
  uint8_t b[100];
  error_t err = ERR_OK;

  err = cmdGet(cmd, b, 100);
  if (err == ERR_OK) {

    if (strcmp(RESP_OK, (char*)b) != 0) {
      // didn't receive OK
      err = ERR_RF_CMD_ERROR;
    }
  }

  return err;
}

/******************************************************************************
 *
 * Description:
 *   Wait for a specific message
 *
 * Params:
 *   [in] msg - msg to wait for
 *   [in] timeout - maximum number of milliseconds to wait
 *
 * Returns:
 *   ERR_OK or an error code if the command failed
 *
 *****************************************************************************/
static error_t waitForMsg(char* msg, uint32_t timeout)
{
  uint32_t t = 0;
  int32_t lineLen = 0;
  uint8_t found = 0;
  char buf[100];
  error_t err = ERR_TIMEOUT;

  t = bsp_timeGet() + timeout;


  while (!found && t > bsp_timeGet()) {

    lineLen = readLine((uint8_t*)buf, 100, 500);

    if (lineLen > 0) {
      if (strcmp(msg, buf) == 0) {
        found = 1;
        err = ERR_OK;
        break;
      }
    }

  }

  return err;
}

/******************************************************************************
 *
 * Description:
 *   Run sequence to reset the module
 *
 *****************************************************************************/
static void resetModule(void)
{
  // PIN12 connected to N_RST. Reset the Xbee module by
  // toggling this pin low->high
  bsp_pinDir(12, 1);

  // reset pulse must be at least 160 us. Using 1 ms.
  bsp_pinClr(12);
  bsp_delayMs(1);
  bsp_pinSet(12);

}

/******************************************************************************
 * Public functions
 *****************************************************************************/


/******************************************************************************
 *
 * Description:
 *   Initialize the Roving RN-XV module. This implementation will always
 *   use UDP as protocol.
 *
 * Params:
 *   [in] ssid - the SSID to use. Must be the same for all devices that should
 *               be able to communicate with each other.
 *   [in] srcIp - IP address of device (must be 4 bytes)
 *   [in] srcPort - local port number for incoming messages
 *   [in] destIp - IP address of remote device (must be 4 bytes)
 *   [in] destPort - remote port number. Messages are sent to this port
 *   [in] mask - IP address mask used by device (must be 4 bytes)
 *
 * Returns:
 *   ERR_OK or an error code if initialization failed
 *
 *****************************************************************************/
error_t rnxv_init(uint8_t* ssid, uint8_t* srcIp, uint16_t srcPort,
    uint8_t* destIp, uint16_t destPort, uint8_t* mask)
{
  error_t err = ERR_OK;
  char cmdBuf[100];
  bsp_comm_arg_t args;

  do {

    args.arg1 = 9600;
    args.arg2 = BSP_ARG2_UART_DATABIT_8;
    args.arg3 = BSP_ARG3_UART_STOPBIT_1;
    args.arg4 = BSP_ARG4_UART_PARITY_NONE;

    bsp_rf_init(BSP_COMM_UART, &args);


    resetModule();

    // wait to make sure module is ready to go to command mode
    bsp_delayMs(1500);


    // go to command mode
    err = commandMode();
    if (err != ERR_OK) {
      dbg("RN-XV: $$$ failed\r\n");
      break;
    }

    // disable echo mode
    cmdGet((uint8_t*)"set uart mode 0x00", (uint8_t*)cmdBuf, 50);
    bsp_delayMs(200);
    bsp_rf_clearRx();

    // set SSID
    sprintf(cmdBuf, "set wlan ssid %s", ssid);
    err = cmdSet((uint8_t*)cmdBuf);
    if (err != ERR_OK) {
      dbg("RN-XV: set ssid failed\r\n");
      break;
    }

    // disable DHCP (use static addresses)
    err = cmdSet((uint8_t*)"set ip dhcp 0");
    if (err != ERR_OK) {
      dbg("RN-XV: disable DHCP failed\r\n");
      break;
    }

    // set the IP protocol (we always use UDP here)
    err = cmdSet((uint8_t*)"set ip protocol 0x01");
    if (err != ERR_OK) {
      dbg("RN-XV: set IP protocol (UDP)\r\n");
      break;
    }

    // set IP address of the module
    sprintf(cmdBuf, "set ip address %d.%d.%d.%d", srcIp[0],srcIp[1],
        srcIp[2],srcIp[3]);
    err = cmdSet((uint8_t*)cmdBuf);
    if (err != ERR_OK) {
      dbg("RN-XV: set IP failed\r\n");
      break;
    }

    // set the local port number
    sprintf(cmdBuf, "set ip localport %d", srcPort);
    err = cmdSet((uint8_t*)cmdBuf);
    if (err != ERR_OK) {
      dbg("RN-XV: set local port failed\r\n");
      break;
    }

    // set IP address of remote module
    sprintf(cmdBuf, "set ip host %d.%d.%d.%d", destIp[0],destIp[1],
        destIp[2],destIp[3]);
    err = cmdSet((uint8_t*)cmdBuf);
    if (err != ERR_OK) {
      dbg("RN-XV: set remote IP failed\r\n");
      break;
    }

    // set the remote port number
    sprintf(cmdBuf, "set ip remote %d", destPort);
    err = cmdSet((uint8_t*)cmdBuf);
    if (err != ERR_OK) {
      dbg("RN-XV: set remote port failed\r\n");
      break;
    }

    // set the network mask
    sprintf(cmdBuf, "set ip netmask %d.%d.%d.%d", mask[0],mask[1],mask[2],mask[3]);
    err = cmdSet((uint8_t*)cmdBuf);
    if (err != ERR_OK) {
      dbg("RN-XV: set mask failed\r\n");
      break;
    }


    // save configuration changes
    cmdSet((uint8_t*)"save");
    // reboot module for changes to take effect
    cmdSet((uint8_t*)"reboot");

    // wait for associated with access point
    err = waitForMsg("Associated!", 10000);
    if (err != ERR_OK) {
      dbg("RN-XV: failed to Associate with AP\r\n");
      break;
    }


  } while (0);


  return err;
}


/******************************************************************************
 *
 * Description:
 *   Receive data from the RF module.
 *
 * Params:
 *   [in] buf - receive buffer
 *   [in] len - size of receive buffer
 *
 * Returns:
 *   number of read bytes (0 if nothing was received)
 *
 *****************************************************************************/
uint32_t rnxv_receive(uint8_t* buf, uint32_t len)
{
  return bsp_rf_receive(buf, len);
}

/******************************************************************************
 *
 * Description:
 *   Send data to the RF module.
 *
 * Params:
 *   [in] txbuf - buffer with data to send
 *   [in] buflen - size of the buffer
 *
 * Returns:
 *   number of written bytes
 *
 *****************************************************************************/
uint32_t rnxv_send(uint8_t *txbuf, uint32_t buflen)
{
  return bsp_rf_send(txbuf, buflen, 1);
}



