/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "lpc_types.h"

#include "bsp.h"
#include "eadebug.h"

#include "rs220x.h"

/******************************************************************************
 * Forward declarations
 *****************************************************************************/

static void rs220xData(uint8_t* ip, uint16_t destPort, uint8_t* buf,
    uint32_t len);


/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/


/*
 * Application configuration
 *
 *
 * CFG_SSID: The SSID (identifier) of the WLAN network. Must be the same for all
 *   devices on the same network
 *
 * CFG_SEC_KEY: Set the security key (password) to use when security (WEP, WPA,
 *   or WPA2) is enabled. Don't define/set this config variable if security
 *   shouldn't be enabled.
 */
#define CFG_SSID     ("EAEA")

#define CFG_SEC_KEY ("mySecKey")

// RF message IDs
#define RFPT_SET_LED (1)

/******************************************************************************
 * Global Variables
 *****************************************************************************/

// buffer used by eadebug.h
uint8_t g_dbbuf[512];

/******************************************************************************
 * Local Variables
 *****************************************************************************/

/*
 * IP Addresses:
 *
 *  - Make sure the IP addresses are valid and unique on your network.
 *  - If this application is downloaded to more than one board the IP
 *    address must be changed. The same IP address can not be used on
 *    several boards connected to the same network.
 *
 */


static uint8_t ip[4]       = {192,168,5,242};
static uint8_t destIp[4]   = {192,168,5,243};

// network mask
static uint8_t mask[4] = {255,255,255,0};

// gateway address
static uint8_t gateway[4]  = {192,168,5,1};

// the same port number is used as destination and source port
static uint16_t commPort   = 0x2616;

#ifdef CFG_SEC_KEY
// set which encryption protocol to use (only valid for infrastructure)
static rs220xEnc_t encProtocol = RS220X_ENC_WPA;
#endif



static rs220x_callb_t callbacks = {
    rs220xData
};


/******************************************************************************
 * Local Functions
 *****************************************************************************/

/******************************************************************************
 *
 * Description:
 *   Received data callback. Called when data has been received by the RF
 *   module.
 *
 * Params:
 *   [in] ip - IP address of the sender (4 bytes)
 *   [in] port - port number of the sender
 *   [in] buf - buffer containing data
 *   [in] len - number of received bytes
 *
 *****************************************************************************/
static void rs220xData(uint8_t* ip, uint16_t port, uint8_t* buf,
    uint32_t len)
{
#ifdef DEBUG
  int i = 0;
  dbg("rs220xData: %d.%d.%d.%d : %d, len=%d\r\n",
      ip[0], ip[1], ip[2], ip[3], port, len);
#endif

  if (len < 1) {
    return;
  }

  switch (buf[0]) {

  // Set LED request. This is a two byte request where the data
  // indicates if the LED should be turned on or off.
  case RFPT_SET_LED:
    if (len > 1) {
      bsp_setLed(buf[1] != 0);
    }
    break;

  default:
#ifdef DEBUG
    for (i = 0; i < len; i++) {
      if (i > 0 && (i%8) == 0) {
        dbg("\r\n");
      }
      dbg("%x ", buf[i]);
    }
    dbg("\r\n");

#endif
    break;
  }



}

/******************************************************************************
 *
 * Description:
 *   Send a 'set LED' request to destination IP address
 *
 * Params:
 *   [in] ledOn - 1 if LED should be turned on; otherwise 0
 *
 *****************************************************************************/
static error_t sendSetLedRequest(uint8_t ledOn)
{
  uint8_t data[2];

  data[0] = RFPT_SET_LED;
  data[1] = ledOn;

  rs220x_send(data, 2);

  return ERR_OK;
}



/******************************************************************************
 * Main method
 *****************************************************************************/

int main (void)
{
  error_t err = ERR_OK;
  uint8_t state = 0;
  uint8_t oldState = 0;


  bsp_init();

  dbg("RS-220X demo (%d.%d.%d.%d)\r\n", ip[0], ip[1], ip[2], ip[3]);
  dbg(" - Infrastructure\r\n");

#ifdef CFG_SEC_KEY
  // enable security. Note that this must be done before xbee_s6_init is called
  err = rs220x_enableEncryption(encProtocol, (uint8_t*)CFG_SEC_KEY);
  if (err != ERR_OK) {
    dbg("Error: Failed to enable encryption\r\n");
    return 0;
  }
#endif

  err = rs220x_init(&callbacks, (uint8_t*)CFG_SSID, ip, commPort, destIp,
      commPort, mask, gateway);
  if (err != ERR_OK) {
    dbg("Error: Failed to initialize RS-220X module\r\n");
    return 0;
  }

  dbg("RS-220X initialization successful\r\n");


  while (1) {

    rs220x_task();

    // check button state
    state = bsp_getButtonState();
    if (oldState != state) {
      oldState = state;

      dbg("Button: %d\r\n", state);

      sendSetLedRequest(state);
    }

  }


  return 1;
}



