/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include <string.h>
#include <stdio.h>

#include "lpc_types.h"

#include "eadebug.h"
#include "bsp.h"
#include "rs220x.h"

/******************************************************************************
 * Typdefs and defines
 *****************************************************************************/

#define RS220X_END_CMD ("\r\n")
#define CR (0x0D)
#define LF (0x0A)
#define RESP_OK ("OK")


#define ERROR_BUF_TOO_SMALL (-1)
#define ERROR_TIMEOUT       (-2)

#define RS220X_BUF_SZ (200)

#define RS220X_MSG_RCV     ("AT+RSI_READ")
#define RS220X_MSG_RCV_LEN (11)


typedef enum {
  STATE_DATA = 0,
  STATE_CR
} rfState_t;

/******************************************************************************
 * Local variables
 *****************************************************************************/

static uint32_t udpSockHandle = 0;

static uint8_t rfBuf[RS220X_BUF_SZ];
static rfState_t rfState = STATE_DATA;
static uint32_t rfPos = 0;

static rs220x_callb_t* _cb = NULL;

static uint8_t* securityKey = NULL;
static rs220xEnc_t encProtocol = RS220X_ENC_WPA2;

/******************************************************************************
 * Local functions
 *****************************************************************************/


/******************************************************************************
 *
 * Description:
 *   Read data from UART up until CR is found.
 *
 * Params:
 *   [in] buf - read data is copied to this buffer
 *   [in] bufLen - size of buffer
 *   [in] timeout - max number of ms to wait for data
 *
 * Returns:
 *   number of read bytes or a negative number if an error occurs
 *
 *****************************************************************************/
static int32_t readLine(uint8_t* buf, uint32_t bufLen, uint32_t timeout)
{
  uint32_t pos = 0;
  uint32_t len = 0;
  uint32_t t = 0;
  uint8_t data;
  uint8_t startFound = 0;

  t = bsp_timeGet() + timeout;


  while(pos < bufLen && t > bsp_timeGet()) {

    len = bsp_rf_receive(&data, 1);

    if (len > 0) {

      // wait until non CR LF is found
      if (!startFound) {
        if (data == LF || data == CR) {
          continue;
        }

        startFound = 1;
      }

      buf[pos] = data;

      if (buf[pos] == LF) {
        buf[pos] = '\0';
        if (pos > 1 && buf[pos-1] == CR) {
          buf[pos-1] = '\0';
          pos--;
        }
        break;
      }

      pos += len;

    }

  }

  if (pos >= bufLen) {
    return ERROR_BUF_TOO_SMALL;
  }

  if (t <= bsp_timeGet()) {
    return ERROR_TIMEOUT;
  }

  return pos;
}



/******************************************************************************
 *
 * Description:
 *   AT command that expects a single line response
 *
 * Params:
 *   [in] atCmd - the AT command string
 *   [in] resp - response buffer
 *   [in] respLen - size of response buffer
 *
 * Returns:
 *   ERR_OK or an error code if the command failed
 *
 *****************************************************************************/
static error_t cmdGet(uint8_t* cmd, uint8_t* resp, uint32_t respLen,
    uint32_t timeout)
{

  int32_t lineLen = 0;
  error_t err = ERR_OK;

  bsp_rf_clearRx();
  bsp_rf_sendString(cmd);
  bsp_rf_sendString((uint8_t*)RS220X_END_CMD);

  bsp_delayMs(450);

  do {

    // a response is expected
    if (resp != NULL && respLen > 0) {
      lineLen = readLine(resp, respLen, timeout);

      if (lineLen < 0) {
        // error while reading
        err = ERR_RF_READ_ERROR;
        break;
      }

      // check if the command is echoed
      if (strncmp((char*)cmd, (char*)resp, strlen((char*)cmd)) == 0) {
        // skip the echoed command and read one more line

        lineLen = readLine(resp, respLen, 1000);

        if (lineLen < 0) {
          // error while reading
          err = ERR_RF_READ_ERROR;
          break;
        }


      }

    }

  } while(0);

  return err;
}

/******************************************************************************
 *
 * Description:
 *   AT command that sets a value and only expects OK or ERROR as result
 *
 * Params:
 *   [in] atCmd - the AT command string
 *
 * Returns:
 *   ERR_OK or an error code if the command failed
 *
 *****************************************************************************/
static error_t cmdSet(uint8_t*cmd, uint8_t* respBuf, uint8_t bufLen,
    uint8_t* respLen, uint32_t timeout)
{
  uint8_t b[100];
  error_t err = ERR_OK;
  uint32_t bLen = 0;
  uint32_t okLen = 0;

  err = cmdGet(cmd, b, 100, timeout);
  if (err == ERR_OK) {

    okLen = strlen(RESP_OK);

    bLen = strlen((char*)b);
    if (strncmp(RESP_OK, (char*)b, okLen) != 0) {
      // didn't receive OK
      err = ERR_RF_CMD_ERROR;
    }

    if (err == ERR_OK && bufLen > 0 && bLen > okLen) {

      memset(respBuf, 0, bufLen);

      if (bLen > bufLen) {
        bLen = bufLen-1;
      }
      memcpy(respBuf, &b[okLen], bLen-okLen);
      *respLen = bLen-okLen;
    }

  }

  return err;
}


/******************************************************************************
 *
 * Description:
 *   Wait for a specific message
 *
 * Params:
 *   [in] msg - msg to wait for
 *   [in] timeout - maximum number of milliseconds to wait
 *
 * Returns:
 *   ERR_OK or an error code if the command failed
 *
 *****************************************************************************/
static error_t waitForMsg(char* msg, uint32_t timeout)
{
  uint32_t t = 0;
  uint32_t len = 0;
  error_t err = ERR_TIMEOUT;
  uint8_t cmpPos = 0;
  uint8_t data;
  uint8_t msgLen = 0;

  msgLen = strlen(msg);
  t = bsp_timeGet() + timeout;

  while (t > bsp_timeGet()) {

    len = bsp_rf_receive(&data, 1);

    if (len > 0 && cmpPos < msgLen) {
      if (data == msg[cmpPos]) {
        cmpPos++;
      }
      else {
        cmpPos = 0;
      }
    }

    if (cmpPos == msgLen) {
      err = ERR_OK;
      break;
    }

  }

  return err;
}


/******************************************************************************
 *
 * Description:
 *   Do a power cycle of the RF module.
 *
 *****************************************************************************/
static void rp_power_cycle(void)
{
  // power enable pin
  bsp_pinDir(11, 1);
  // reset pin
  bsp_pinDir(12, 1);

  // power off
  bsp_pinClr(11);

  // reset and wait 10 ms
  bsp_pinClr(12);
  bsp_delayMs(10);

  // power on
  bsp_pinSet(11);

  // let power stabilize
  bsp_delayMs(20);

  // release reset
  bsp_pinSet(12);

}

/******************************************************************************
 *
 * Description:
 *   Initiate auto baud rate detection.
 *
 *****************************************************************************/
static error_t rp_abrd(void)
{
  uint8_t data;
  uint32_t t = 0;
  uint8_t sent_1c = 0;
  error_t err = ERR_RF_ABRD_FAILED;
  uint32_t len = 0;

  /*
   * Auto Baud Rate Detection
   *
   * 1. Host sends 0x1C
   * 2. Wait to receive 0x55. If not received within 200 ms resend 0x1C
   * 3. When receiving 0x55 sent 0x55.
   */

  do {

    if (sent_1c >=2 && t + 200 < bsp_timeGet()) {
      // ABRD failed
      break;
    }

    if (sent_1c < 2 && t + 200 < bsp_timeGet()) {
      data = 0x1C;
      bsp_rf_send(&data, 1, 1);
      t = bsp_timeGet();
      sent_1c++;
    }

    if (sent_1c > 0) {
      len = bsp_rf_receive(&data, 1);
      if (len > 0 && data == 0x55) {
        bsp_rf_send(&data, 1, 1);
        err = ERR_OK;
      }
    }

  } while(1);


  return err;
}

/******************************************************************************
 *
 * Description:
 *   Process a received frame of data
 *
 * Params:
 *   [in] buf - buffer containing data
 *   [in] bufLen - number of bytes in buffer
 *
 *****************************************************************************/
static void processFrame(uint8_t* buf, uint32_t bufLen)
{
  uint16_t sz = 0;
  uint8_t* ip;
  uint16_t port;

  if (bufLen > RS220X_MSG_RCV_LEN &&
      strncmp(RS220X_MSG_RCV, (char*)buf, RS220X_MSG_RCV_LEN) == 0) {

    buf += RS220X_MSG_RCV_LEN;

    // Receive data on socket

    // (1)  hn - handle
    // (2)  sz - size
    // (4)  sip - source IP address
    // (2)  sport - source port
    // (sz) stream  - stream of data

    if (bufLen > RS220X_MSG_RCV_LEN + 9) {
      sz = (buf[1] | (buf[2] << 8));
      ip = &buf[3];
      port = (buf[7] | (buf[8] << 8));

      if (_cb != NULL && sz <= (bufLen-RS220X_MSG_RCV_LEN-9)) {
        _cb->data(ip, port, &buf[9], sz);
      }
    }

  }

}

/******************************************************************************
 *
 * Description:
 *   Process one byte of data received from the XBee module
 *
 * Params:
 *   [in] data - the received data
 *
 *****************************************************************************/
static void processByte(uint8_t data)
{
  switch(rfState) {
  case STATE_DATA:

    if (rfPos >= RS220X_BUF_SZ) {
      dbg("RS-220X: Frame len %d > max buffer len %d\r\n", rfPos, RS220X_BUF_SZ);
      rfPos = 0;
      break;
    }

    if (data == CR) {
      rfState = STATE_CR;
    }
    else {
      rfBuf[rfPos++] = data;
    }

    break;
  case STATE_CR:
    // wait for LF
    if (data == LF) {
      processFrame(rfBuf, rfPos);
    }
    else {
      dbg("RS-220X: Invalid data recvd when waiting for LF\r\n");
    }

    rfState = STATE_DATA;
    rfPos = 0;

    break;

  }
}


/******************************************************************************
 * Public functions
 *****************************************************************************/


/******************************************************************************
 *
 * Description:
 *   Initialize the Red Signals RS-XMEGA-220X module.
 *
 * Params:
 *   [in] ssid - the SSID to use. Must be the same for all devices that should
 *               be able to communicate with each other.
 *   [in] srcIp - IP address of device (must be 4 bytes)
 *   [in] srcPort - local port number for incoming messages
 *   [in] destIp - IP address of remote device (must be 4 bytes)
 *   [in] destPort - remote port number. Messages are sent to this port
 *   [in] mask - IP address mask used by device (must be 4 bytes)
 *   [in] gw - gateway IP address (must be 4 bytes)
 *
 * Returns:
 *   ERR_OK or an error code if initialization failed
 *
 *****************************************************************************/
error_t rs220x_init(rs220x_callb_t* cb, uint8_t* ssid, uint8_t* srcIp,
    uint16_t srcPort, uint8_t* destIp, uint16_t destPort, uint8_t* mask,
    uint8_t* gw)
{
  error_t err = ERR_OK;
  bsp_comm_arg_t args;

  char cmdBuf[100];
  char respBuf[30];
  uint8_t respLen = 0;

  _cb = cb;

  do {

    args.arg1 = 115200;
    args.arg2 = BSP_ARG2_UART_DATABIT_8;
    args.arg3 = BSP_ARG3_UART_STOPBIT_2;
    args.arg4 = BSP_ARG4_UART_PARITY_NONE;

    bsp_rf_init(BSP_COMM_UART, &args);


    // power cycle the module
    rp_power_cycle();

    // wait 100 ms before starting auto baud rate detection
    bsp_delayMs(100);

    // auto baud rate detection
    err = rp_abrd();
    if (err != ERR_OK) {
      dbg("RS-220X: ABRD failed\r\n");
      break;
    }

    // wait for the firmware to have loaded in the module
    err = waitForMsg(" Loading Done\r\n", 6000);
    if (err != ERR_OK) {
      dbg("RS-220X: Expected msg not rcvd\r\n");
      break;
    }

    // select which band to use
    err = cmdSet((uint8_t*)"at+rsi_band=0", NULL, 0, NULL, 1000);
    if (err != ERR_OK) {
      dbg("RS-220X: Select band failed\r\n");
      break;
    }

    // init the module
    err = cmdSet((uint8_t*)"at+rsi_init", NULL, 0, NULL, 1000);
    if (err != ERR_OK) {
      dbg("RS-220X: Init module failed\r\n");
      break;
    }

    // do a scan (this must be done before a join)
    sprintf(cmdBuf, "at+rsi_scan=0,%s", ssid);
    err = cmdSet((uint8_t*)cmdBuf, NULL, 0, NULL, 5000);
    if (err != ERR_OK) {
      dbg("RS-220X: Scan failed\r\n");
      break;
    }

    // select network type
    err = cmdSet((uint8_t*)"at+rsi_network=INFRASTRUCTURE",
        NULL, 0, NULL, 1000);
    if (err != ERR_OK) {
      dbg("RS-220X: Set network type failed\r\n");
      break;
    }

    if (securityKey != NULL) {

      if (encProtocol == RS220X_ENC_WEP) {

        err = cmdSet((uint8_t*)"at+rsi_authmode=1",
            NULL, 0, NULL, 1000);
        if (err != ERR_OK) {
          dbg("RS-220X: Set auth mode failed\r\n");
          break;
        }

      }

      sprintf(cmdBuf, "at+rsi_psk=%s", securityKey);
      err = cmdSet((uint8_t*)cmdBuf, NULL, 0, NULL, 1000);
      if (err != ERR_OK) {
        dbg("RS-220X: Set Pre-shared key failed\r\n");
        break;
      }

    }

    // select network type
    sprintf(cmdBuf, "at+rsi_join=%s,0,0", ssid);
    err = cmdSet((uint8_t*)cmdBuf, NULL, 0, NULL, 10000);
    if (err != ERR_OK) {
      dbg("RS-220X: Join failed\r\n");
      break;
    }

    // configure IP
    sprintf(cmdBuf, "at+rsi_ipconf=0,%d.%d.%d.%d,%d.%d.%d.%d,%d.%d.%d.%d",
        srcIp[0],srcIp[1],srcIp[2],srcIp[3],
        mask[0],mask[1],mask[2],mask[3],
        gw[0],gw[1],gw[2],gw[3]);
    err = cmdSet((uint8_t*)cmdBuf, NULL, 0, NULL, 1000);
    if (err != ERR_OK) {
      dbg("RS-220X: IP config failed\r\n");
      break;
    }

    // Open a UDP socket
    sprintf(cmdBuf, "at+rsi_udp=%d.%d.%d.%d,%d,%d",
        destIp[0],destIp[1],destIp[2],destIp[3],
        destPort, srcPort);
    err = cmdSet((uint8_t*)cmdBuf, (uint8_t*)respBuf, 30, &respLen, 1000);
    if (err != ERR_OK) {
      dbg("RS-220X: Open UDP socket failed\r\n");
      break;
    }

    if (respLen > 0) {
      udpSockHandle = respBuf[0];
    }

  } while (0);


  return err;
}

/******************************************************************************
 *
 * Description:
 *   Enable encryption (only valid when network type is infrastructure).
 *
 *   Note: This function must be called before rs220x_init.
 *
 * Params:
 *   [in] enc - encryption protocol (WPA, WPA2, or WEP)
 *   [in] key - the security key as a null-terminated string.
 *              WEP : may only contain hexadecimal digits (0-F).
 *              Either 10 digits for WEP-64 or 26 digits for WEP-128.
 *              WPA/WPA2 : any combination of ASCII characters up to
 *              63 characters.
 *
 * Returns:
 *   ERR_OK or an error code if failed
 *
 *****************************************************************************/
error_t rs220x_enableEncryption(rs220xEnc_t enc, uint8_t* key)
{
  uint32_t keyLen = 0;

  if (key == NULL) {
    return ERR_ARGUMENT;
  }

  keyLen = strlen((char*)key);

  // key length must be 10 or 26 hexadecimal digits (64 or 128 bits) when using WEP)
  if (enc == RS220X_ENC_WEP && keyLen != 10 && keyLen != 26) {
    return ERR_ARGUMENT;
  }

  // not accepting zero length key for WPA and WPA2 or keys > 63 characters
  if ((enc == RS220X_ENC_WPA || enc == RS220X_ENC_WPA2)
      && (keyLen == 0 || keyLen > 63)) {
    return ERR_ARGUMENT;
  }

  securityKey = key;
  encProtocol = enc;

  return ERR_OK;
}


/******************************************************************************
 *
 * Description:
 *   Send data to the RF module.
 *
 * Params:
 *   [in] txbuf - buffer with data to send
 *   [in] buflen - size of the buffer
 *
 * Returns:
 *   number of written bytes
 *
 *****************************************************************************/
uint32_t rs220x_send(uint8_t *txbuf, uint32_t bufLen)
{

  char cmdBuf[50];
  uint32_t okLen = 0;
  int32_t lineLen = 0;
  uint32_t sent = 0;

  sprintf(cmdBuf, "at+rsi_snd=%d,%d,0,0,",udpSockHandle,bufLen);
  bsp_rf_send((uint8_t*)cmdBuf, strlen(cmdBuf), 1);
  bsp_rf_send(txbuf, bufLen, 1);
  bsp_rf_send((uint8_t*)"\r\n", 2, 1);

  do {

      lineLen = readLine((uint8_t*)cmdBuf, 50, 1000);

      if (lineLen < 0) {
        // error while reading
        break;
      }

      okLen = strlen(RESP_OK);
      if (strncmp(RESP_OK, (char*)cmdBuf, okLen) != 0) {
        // didn't receive OK
        break;
      }

      lineLen -= okLen;
      if (lineLen == 2) {
        sent = ((cmdBuf[okLen+1] << 8) | (cmdBuf[okLen]));
      }


  } while(0);


  return sent;
}

/******************************************************************************
 *
 * Description:
 *   Call this function repeatedly to run the driver
 *
 *****************************************************************************/
void rs220x_task(void)
{
  uint32_t len;
  uint8_t data;

  if (!bsp_rf_recvIsEmpty()) {

    len = bsp_rf_receive(&data, 1);
    if (len > 0) {
      processByte(data);
    }

  }

}



