/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "lpc_types.h"

#include "bsp.h"
#include "eadebug.h"

#include "xbee.h"

/******************************************************************************
 * Forward declarations
 *****************************************************************************/

static void xbeeUp(uint8_t up);
static void xbeeNode(uint32_t addrHi, uint32_t addrLo, uint8_t rssi);
static void xbeeTxStatus(uint8_t frameId, xbeeTxStatus_t error);
static void xbeeData(uint32_t addrHi, uint32_t addrLo, uint8_t rssi,
    uint8_t* buf, uint8_t len);

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

/*
 * Application configuration
 *
 * CFG_ACT_AS_COORDINATOR - Configure the XBee module to act as a coordinator.
 *                      If undefined the module will be an end-device
 */
#define CFG_ACT_AS_COORDINATOR (1)


// RF message IDs
#define RFPT_SET_LED (1)

/******************************************************************************
 * Global Variables
 *****************************************************************************/

uint8_t g_dbbuf[512];

/******************************************************************************
 * Local Variables
 *****************************************************************************/

static xbee_callb_t callbacks = {
    xbeeUp,
    xbeeNode,
    xbeeTxStatus,
    xbeeData
};

static uint8_t devIsReady = 0;


/******************************************************************************
 * Local Functions
 *****************************************************************************/

/******************************************************************************
 *
 * Description:
 *   XBee node up/down callback
 *
 * Params:
 *   [in] up - 1 if the node is up, 0 if it is down
 *
 *****************************************************************************/
static void xbeeUp(uint8_t up)
{
  dbg("RF: Xbee Up (%d)\r\n", up);
  devIsReady = up;
}

/******************************************************************************
 *
 * Description:
 *   XBee node discover callback. Will be called as a response to a Xbee node
 *   discovery request. All found nodes are reported back one-by-one
 *   through this callback.
 *
 * Params:
 *   [in] addrHi - upper 32 bits of the 64-bit node address
 *   [in] addrLo - lower 32 bits of the 64-bit node address
 *   [in] rssi - signal strength
 *
 *****************************************************************************/
static void xbeeNode(uint32_t addrHi, uint32_t addrLo, uint8_t rssi)
{
  dbg("RF: Node %x:%x rssi=%d\r\n", addrHi, addrLo, rssi);
}

/******************************************************************************
 *
 * Description:
 *   Transmit status callback. Called as a result of a packet being sent
 *   from the Xbee node.
 *
 * Params:
 *   [in] frameId - ID of the frame that was sent
 *   [in] status - status of the transmit request
 *
 *****************************************************************************/
static void xbeeTxStatus(uint8_t frameId, xbeeTxStatus_t status)
{
  if (status != XBEE_TX_STAT_OK) {
    dbg("RF: [%d] TX failed %d\r\n", frameId, status);
  }
}

/******************************************************************************
 *
 * Description:
 *   Received data callback. Called when data has been received by the Xbee
 *   node.
 *
 * Params:
 *   [in] addrHi - upper 32 bits of the 64-bit node address
 *   [in] addrLo - lower 32 bits of the 64-bit node address
 *   [in] rssi - signal strength
 *   [in] buf - buffer containing the data
 *   [in] len - number of received bytes
 *
 *****************************************************************************/
static void xbeeData(uint32_t addrHi, uint32_t addrLo, uint8_t rssi,
    uint8_t* buf, uint8_t len)
{
#ifdef DEBUG
  int i = 0;
  dbg("xbeeData %x:%x, rssi=%d, len=%d\r\n", addrHi, addrLo, rssi, len);
#endif

  if (len < 1) {
    return;
  }

  switch (buf[0]) {

  // Set LED request. This is a two byte request where the data
  // indicates if the LED should be turned on or off.
  case RFPT_SET_LED:
    if (len > 1) {
      bsp_setLed(buf[1] != 0);
    }
    break;

  default:
#ifdef DEBUG
    for (i = 0; i < len; i++) {
      if (i > 0 && (i%8) == 0) {
        dbg("\r\n");
      }
      dbg("%x ", buf[i]);
    }
    dbg("\r\n");

#endif
    break;
  }


}

static error_t sendSetLedRequest(uint8_t ledOn)
{
  uint8_t data[2];
  uint8_t id = 0;

  data[0] = RFPT_SET_LED;
  data[1] = ledOn;

  return xbee_send(XBEE_ADDRHI_BROADCAST, XBEE_ADDRLO_BROADCAST, data, 2, &id);
}

/******************************************************************************
 * Main method
 *****************************************************************************/

int main (void)
{
  error_t err = ERR_OK;
  uint8_t state = 0;
  uint8_t oldState = 0;


  bsp_init();

  dbg("XBee demo\r\n");

#if (CFG_ACT_AS_COORDINATOR == 1)
  err = xbee_init(XBEE_COORDINATOR, &callbacks);
#else
  err = xbee_init(XBEE_END_DEVICE, &callbacks);
#endif

  while (1) {
    xbee_task();

    if (devIsReady != 0) {
      // check button state
      state = bsp_getButtonState();
      if (oldState != state) {
        oldState = state;
        dbg("Button: %d\r\n", state);

        sendSetLedRequest(state);
      }
    }


  }

  return 1;
}



