/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "lpc_types.h"

#include "bsp.h"
#include "eadebug.h"

#include "xbee_s6.h"

/******************************************************************************
 * Forward declarations
 *****************************************************************************/

static void xbeeUp(uint8_t up);
static void xbeeTxStatus(uint8_t frameId, xbeeS6TxStatus_t error);
static void xbeeData(uint8_t* ip, uint16_t destPort, uint16_t srcPort,
    xbeeS6Protocol_t protocol, uint8_t* buf, uint32_t len);

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

#define CFG_VAL_INFRASTRUCTURE (1)
#define CFG_VAL_ADHOC_CREATOR  (2)
#define CFG_VAL_ADHOC_JOINER   (3)

/*
 * Application configuration
 *
 * CFG_NET_TYPE - decide the net type for the application. This can be either
 *   CFG_VAL_INFRASTRUCTURE: Connect to a Access Point
 *   CFG_VAL_ADHOC_CREATOR:  Creator for Ad Hoc network
 *   CFG_VAL_ADHOC_JOINER:   Joiner for Ad Hoc network
 *
 * CFG_SSID: The SSID (identifier) of the WLAN network. Must be the same for all
 *   devices on the same network
 * CFG_CHANNEL: The channel to use
 *
 * CFG_SEC_KEY: Set the security key (password) to use when security (WEP, WPA,
 *   or WPA2) is enabled. Don't define/set this config variable if security
 *   shouldn't be enabled.
 */
#define CFG_NET_TYPE (/*CFG_VAL_ADHOC_CREATOR*/CFG_VAL_INFRASTRUCTURE)
#define CFG_SSID     ("EAEA")
#define CFG_CHANNEL  (6)

#define CFG_SEC_KEY ("mySecKey")

// RF message IDs
#define RFPT_SET_LED (1)

/******************************************************************************
 * Global Variables
 *****************************************************************************/

uint8_t g_dbbuf[512];

/******************************************************************************
 * Local Variables
 *****************************************************************************/

/*
 * IP Addresses:
 *
 * Infrastructure
 *  - Make sure the IP addresses are valid and unique on your network.
 *  - If this application is downloaded to more than one board the IP
 *    address must be changed. The same IP address can not be used on
 *    several boards connected to the same network.
 *
 * Ad Hoc
 *  - Communication is always between two nodes and not connected to
 *    any network. All that is needed is that the IP addresses are
 *    unique on each board (handled by the configuration below)
 */


#if (CFG_NET_TYPE == CFG_VAL_INFRASTRUCTURE) || (CFG_NET_TYPE == CFG_VAL_ADHOC_CREATOR)
static uint8_t ip[4]       = {192,168,5,/*242*/243};
static uint8_t destIp[4]   = {192,168,5,/*243*/242};
#else
static uint8_t ip[4]       = {192,168,5,243};
static uint8_t destIp[4]   = {192,168,5,242};
#endif

// network mask
static uint8_t mask[4] = {255,255,255,0};

// the same port number is as destination and source port
static uint16_t commPort   = 0x2616;

#ifdef CFG_SEC_KEY
// set which encryption protocol to use (only valid for infrastructure)
static xbeeS6Enc_t encProtocol = XBEE_S6_ENC_WPA;
#endif

static xbee_s6_callb_t callbacks = {
    xbeeUp,
    xbeeTxStatus,
    xbeeData
};

static uint8_t devIsReady = 0;


/******************************************************************************
 * Local Functions
 *****************************************************************************/

/******************************************************************************
 *
 * Description:
 *   XBee node up/down callback
 *
 * Params:
 *   [in] up - 1 if the node is up, 0 if it is down
 *
 *****************************************************************************/
static void xbeeUp(uint8_t up)
{
  dbg("RF: Xbee Up (%d)\r\n", up);
  devIsReady = up;
}

/******************************************************************************
 *
 * Description:
 *   Transmit status callback. Called as a result of a packet being sent
 *   from the Xbee node.
 *
 * Params:
 *   [in] frameId - ID of the frame that was sent
 *   [in] status - status of the transmit request
 *
 *****************************************************************************/
static void xbeeTxStatus(uint8_t frameId, xbeeS6TxStatus_t status)
{
  if (status != XBEE_S6_TX_STAT_OK) {
    dbg("RF: [%d] TX failed %d\r\n", frameId, status);
  }
}

/******************************************************************************
 *
 * Description:
 *   Received data callback. Called when data has been received by the Xbee
 *   node.
 *
 * Params:
 *   [in] ip - IP address of the sender (4 bytes)
 *   [in] destPort - destination port number
 *   [in] srcPort - source port number
 *   [in] protocol - UDP or TCP
 *   [in] buf - buffer containing data
 *   [in] len - number of received bytes
 *
 *****************************************************************************/
static void xbeeData(uint8_t* ip, uint16_t destPort, uint16_t srcPort,
    xbeeS6Protocol_t protocol, uint8_t* buf, uint32_t len)
{
#ifdef DEBUG
  int i = 0;
  dbg("xbeeData: %d.%d.%d.%d : %d -> %d, prot=%d, len=%d\r\n",
      ip[0], ip[1], ip[2], ip[3], srcPort, destPort, protocol, len);
#endif

  if (len < 1) {
    return;
  }

  switch (buf[0]) {

  // Set LED request. This is a two byte request where the data
  // indicates if the LED should be turned on or off.
  case RFPT_SET_LED:
    if (len > 1) {
      bsp_setLed(buf[1] != 0);
    }
    break;

  default:
#ifdef DEBUG
    for (i = 0; i < len; i++) {
      if (i > 0 && (i%8) == 0) {
        dbg("\r\n");
      }
      dbg("%x ", buf[i]);
    }
    dbg("\r\n");

#endif
    break;
  }


}

/******************************************************************************
 *
 * Description:
 *   Send a 'set LED' request to destination IP address
 *
 * Params:
 *   [in] ledOn - 1 if LED should be turned on; otherwise 0
 *
 *****************************************************************************/
static error_t sendSetLedRequest(uint8_t ledOn)
{
  uint8_t data[2];
  uint8_t id = 0;

  data[0] = RFPT_SET_LED;
  data[1] = ledOn;

  return xbee_s6_send(destIp, commPort, commPort, XBEE_S6_UDP, data, 2, &id);
}

/******************************************************************************
 * Main method
 *****************************************************************************/

int main (void)
{
  error_t err = ERR_OK;
  uint8_t state = 0;
  uint8_t oldState = 0;
  xbeeS6Type_t type;

  bsp_init();

  dbg("XBee demo (%d.%d.%d.%d)\r\n", ip[0], ip[1], ip[2], ip[3]);

#if CFG_NET_TYPE == CFG_VAL_INFRASTRUCTURE
  type = XBEE_S6_INFRASTRUCTURE;
  dbg(" - Infrastructure\r\n");
#elif CFG_NET_TYPE == CFG_VAL_ADHOC_CREATOR
  type = XBEE_S6_ADHOC_CREATOR;
  dbg(" - Ad Hoc Creator\r\n");
#else
  type = XBEE_S6_ADHOC_JOINER;
  dbg(" - Ad Hoc Joiner\r\n");
#endif

#ifdef CFG_SEC_KEY
  // enable security. Note that this must be done before xbee_s6_init is called
  err = xbee_s6_enableEncryption(encProtocol, (uint8_t*)CFG_SEC_KEY);
  if (err != ERR_OK) {
    dbg("Error: Failed to enable encryption\r\n");
    return 0;
  }
#endif


  err = xbee_s6_init(type, &callbacks, (uint8_t*)CFG_SSID, CFG_CHANNEL,
      ip, mask);
  if (err != ERR_OK) {
    dbg("Error: Failed to initialize XBee module\r\n");
    return 0;
  }

  while (1) {
    xbee_s6_task();

    if (devIsReady != 0) {

      // check button state
      state = bsp_getButtonState();
      if (oldState != state) {
        oldState = state;

        dbg("Button: %d\r\n", state);

        sendSetLedRequest(state);
      }
    }

  }

  return 1;
}



