/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include <string.h>
#include <stdio.h>

#include "lpc_types.h"

#include "eadebug.h"
#include "bsp.h"
#include "xbee_s6.h"

/******************************************************************************
 * Typdefs and defines
 *****************************************************************************/

#define XBEE_END_CMD ("\r")
#define CR (0x0D)
#define RESP_OK ("OK")

#define ERROR_BUF_TOO_SMALL (-1)
#define ERROR_TIMEOUT       (-2)

#define XBEE_START_DEL (0x7E)
#define XBEE_BUF_SZ (200)


#define XBEE_API_ID_TX_64       (0x00)
#define XBEE_API_ID_AT_CMD      (0x08)
#define XBEE_API_ID_AT_CMD_Q    (0x09)
#define XBEE_API_ID_REMOTE_CMD  (0x07)
#define XBEE_API_ID_TX_IPV4     (0x20)
#define XBEE_API_ID_RX_64       (0x80)
#define XBEE_API_ID_AT_RESP     (0x88)
#define XBEE_API_ID_TX_STAT     (0x89)
#define XBEE_API_ID_MOD_STAT    (0x8A)
#define XBEE_API_ID_IO_RX_IND   (0x8F)
#define XBEE_API_ID_REMOTE_RESP (0x87)
#define XBEE_API_ID_RX_IPV4     (0xB0)

#define XBEE_MOD_STAT_HW_RESET    (0)
#define XBEE_MOD_STAT_WD_RESET    (1)
#define XBEE_MOD_STAT_ASSOC       (2)
#define XBEE_MOD_STAT_DIASSOC     (3)
#define XBEE_MOD_STAT_IP_CFG      (4)


#define XBEE_TX_STAT_SUCCESS      (0x00)
#define XBEE_TX_STAT_PURGED       (0x03)
#define XBEE_TX_STAT_PHY_ERROR    (0x04)
#define XBEE_TX_STAT_TX64_TIMEOUT (0x21)
#define XBEE_TX_STAT_RCS_ERROR    (0x32)
#define XBEE_TX_STAT_MSG_TOO_LONG (0x74)
#define XBEE_TX_STAT_SOCK_FAILED  (0x76)

#define XBEE_RECV_FRAME_TO (2000)

#define XBEE_SND_BUF_SZ (300)

typedef enum {
  XBEE_FRAME,
  XBEE_LENGTH,
  XBEE_DATA,
} rfState_t;

/******************************************************************************
 * Local variables
 *****************************************************************************/

static xbee_s6_callb_t* _cb = NULL;

static rfState_t rfState = XBEE_FRAME;
static uint32_t rfFrameLen = 0;
static uint32_t rfPos = 0;
static uint8_t rfBuf[XBEE_BUF_SZ];
static uint8_t rfFrameId = 1;
static uint32_t rfFrameTimer = 0;

static uint8_t initialized = 0;

static uint8_t* securityKey = NULL;
static xbeeS6Enc_t encProtocol = XBEE_S6_ENC_WPA2;

static uint8_t sndBuf[XBEE_SND_BUF_SZ];

/******************************************************************************
 * Local functions
 *****************************************************************************/



/******************************************************************************
 *
 * Description:
 *   Get a new frame ID
 *
 * Returns:
 *   New frame ID
 *
 *****************************************************************************/
static uint8_t getFrameId(void)
{
  rfFrameId++;
  if (rfFrameId == 0) {
    rfFrameId = 1;
  }

  return rfFrameId;
}

/******************************************************************************
 *
 * Description:
 *   Calculate checksum
 *
 * Params:
 *   [in] buf - buffer containing data
 *   [in] len - number of bytes
 *
 * Returns:
 *   Calculated checksum
 *
 *****************************************************************************/
static uint8_t checksum(uint8_t* buf, uint32_t len)
{
  int i = 0;
  uint8_t cs = 0;

  for (i = 0; i < len; i++) {
    cs += buf[i];
  }

  return (0xFF - cs);
}

/******************************************************************************
 *
 * Description:
 *   Transmit data to other node using 64-bit address
 *
 * Params:
 *   [in] addrHi - upper 32 bits of 64 bit address
 *   [in] addrLo - lower 32 bits of 64 bit address
 *   [in] data - data to transmit
 *   [in] len - number of bytes
 *   [out] frameId - frame ID of the request
 *
 * Returns:
 *   ERR_OK or an error code if the request failed.
 *
 *****************************************************************************/
static error_t apiTxIPv4(uint8_t* destIp, uint16_t destPort, uint16_t srcPort,
    xbeeS6Protocol_t protocol, uint8_t* data, uint32_t len, uint8_t* frameId)
{
  uint32_t tmp = len+12;


  if (len+16 > XBEE_SND_BUF_SZ) {
    return ERR_ARGUMENT;
  }

  sndBuf[0] = XBEE_START_DEL;

  // length
  sndBuf[1] = ((tmp >> 8) & 0xff);
  sndBuf[2] = (tmp & 0xff);

  // AP ID
  sndBuf[3] = XBEE_API_ID_TX_IPV4;

  // frame ID
  sndBuf[4] = getFrameId();


  // IP address
  sndBuf[5] = destIp[0];
  sndBuf[6] = destIp[1];
  sndBuf[7] = destIp[2];
  sndBuf[8] = destIp[3];

  sndBuf[9]  = ((destPort >> 8) & 0xff);
  sndBuf[10] = (destPort & 0xff);

  sndBuf[11]  = ((srcPort >> 8) & 0xff);
  sndBuf[12] = (srcPort & 0xff);

  sndBuf[13] = protocol;

  // options
  sndBuf[14] = 0;

  // data
  memcpy(&sndBuf[15], data, len);

  // checksum
  sndBuf[15+len] = checksum(&sndBuf[3], tmp);

  if (frameId != NULL) {
    *frameId = sndBuf[4];
  }

  bsp_rf_send(sndBuf, 16+len, BLOCKING);

  return ERR_OK;
}


/******************************************************************************
 *
 * Description:
 *   Handle a TX status event
 *
 * Params:
 *   [in] frameId - ID of the transmitted frame
 *   [in] status - status of the TX
 *
 *****************************************************************************/
static void handleTxStatus(uint8_t frameId, uint8_t status)
{

  if (_cb->txStat != NULL) {
    switch(status)  {
    case XBEE_TX_STAT_SUCCESS:
      _cb->txStat(frameId, XBEE_S6_TX_STAT_OK);
      break;
    case XBEE_TX_STAT_PURGED:
      _cb->txStat(frameId, XBEE_S6_TX_STAT_PURGED);
      break;
    case XBEE_TX_STAT_PHY_ERROR:
      _cb->txStat(frameId, XBEE_S6_TX_STAT_PHY_ERROR);
      break;
    case XBEE_TX_STAT_TX64_TIMEOUT:
      _cb->txStat(frameId, XBEE_S6_TX_STAT_TX64_TIMEOUT);
      break;
    case XBEE_TX_STAT_RCS_ERROR:
      _cb->txStat(frameId, XBEE_S6_TX_STAT_RCS_ERROR);
      break;
    case XBEE_TX_STAT_MSG_TOO_LONG:
      _cb->txStat(frameId, XBEE_S6_TX_STAT_MSG_TOO_LONG);
      break;
    case XBEE_TX_STAT_SOCK_FAILED:
      _cb->txStat(frameId, XBEE_S6_TX_STAT_SOCK_FAIL);
      break;
    }
  }
#ifdef DEBUG
  else {
    dbg("Xbee: Tx status = %d\r\n", status)
  }
#endif
}

/******************************************************************************
 *
 * Description:
 *   Process received data
 *
 * Params:
 *   [in] addrHi - upper 32 bits of 64 bit address
 *   [in] addrLo - lower 32 bits of 64 bit address
 *   [in] rssi - signal strength
 *   [in] buf - recieved data
 *   [in] len - number of bytes
 *
 *****************************************************************************/
static void processData(uint8_t* ip, uint16_t destPort, uint16_t srcPort,
    xbeeS6Protocol_t protocol, uint8_t* buf, uint32_t len)
{


  if (_cb->data != NULL) {
    _cb->data(ip, destPort, srcPort, protocol, buf, len);
  }
#ifdef DEBUG
  else {
    int i = 0;
    sprintf((char*)g_dbbuf, "data: %d.%d.%d.%d : %d -> %d, prot=%d, len=%d\r\n",
        ip[0], ip[1], ip[2], ip[3], srcPort, destPort, protocol, len);
    bsp_console_sendString(g_dbbuf);

    for (i = 0; i < len; i++) {
      sprintf((char*)g_dbbuf, "%x ", buf[i]);
      bsp_console_sendString(g_dbbuf);
    }
    bsp_console_sendString((uint8_t*)"\r\n");
  }
#endif


}

/******************************************************************************
 *
 * Description:
 *   Handle an AT response packet
 *
 * Params:
 *   [in] frameId - frame ID
 *   [in] atBuf - 2 bytes (ASCII) identifying the AT command
 *   [in] status - status of the command
 *   [in] valueBuf - value is stored in this buffer
 *   [in] valueLen - number of bytes in the valueBuf
 *
 *****************************************************************************/
static void handleAtResponse(uint8_t frameId, uint8_t* atBuf, uint8_t status,
    uint8_t* valueBuf, uint32_t valueLen)
{
#ifdef DEBUG

  int i = 0;

  dbg("Xbee: AT response %x, %c%c, stat=%d, len=%d\r\n    ",
      frameId, atBuf[0], atBuf[1], status, valueLen);

  for (i = 0; i < valueLen; i++) {
    sprintf((char*)g_dbbuf, "%x ",
        valueBuf[i]);
    bsp_console_sendString(g_dbbuf);
  }
  bsp_console_sendString((uint8_t*)"\r\n");

#endif

}

/******************************************************************************
 *
 * Description:
 *   Handle modem status
 *
 * Params:
 *   [in] status - modem status
 *
 *****************************************************************************/
static void handleModemStatus(uint8_t status)
{
  dbg("Xbee: Modem status %d\r\n", status)

  if (status == XBEE_MOD_STAT_ASSOC) {
    initialized = 1;
    if (_cb->up != NULL) {
      _cb->up(1);
    }
  }
  else if (status == XBEE_MOD_STAT_DIASSOC) {
    initialized = 0;
    if (_cb->up != NULL) {
      _cb->up(0);
    }
  }

}

/******************************************************************************
 *
 * Description:
 *   Process a received frame (API mode)
 *
 * Params:
 *   [in] buf - buffer containing the frame
 *   [in] len - number of received bytes
 *
 *****************************************************************************/
static void processFrame(uint8_t* buf, uint32_t len)
{
#ifdef DEBUG
  int i = 0;
#endif

  uint8_t* b = NULL;
  uint32_t bLen = 0;
  uint16_t destPort = 0;
  uint16_t srcPort = 0;

  if (len < 2) {
    dbg("Xbee: Invalid frame length (%d)\r\n", len);
    return;
  }

  // verify checksum
  if (checksum(buf, len) != 0) {
    dbg("Xbee: Invalid checksum\r\n");
    return;
  }


  switch(buf[0]) {
  case XBEE_API_ID_AT_RESP:
    if (len < 5) {
      dbg("Xbee: AT resp data too small: %d\r\n ", len);
      return;
    }

    // there is a value
    if (len > 6) {
      b = &buf[5];
      bLen = len-5-1;
    }

    handleAtResponse(buf[1], &buf[2], buf[4], b, bLen);
    break;
  case XBEE_API_ID_TX_STAT:
    handleTxStatus(buf[1], buf[2]);
    break;
  case XBEE_API_ID_RX_64:
    dbg("Xbee: RX 64 bit (unhandled)\r\n");
    break;
  case XBEE_API_ID_RX_IPV4:

    if (len < 12) {
      dbg("Xbee: RX IPv4 len error (%d)\r\n", len);
    }

    destPort = (buf[5] << 8 | buf[6]);
    srcPort = (buf[7] << 8 | buf[8]);

    processData(&buf[1], destPort, srcPort, buf[9], &buf[11], len-11-1);
    break;


  case XBEE_API_ID_MOD_STAT:
    handleModemStatus(buf[1]);
    break;
  default:
    dbg("Xbee: Unhandled API ID: %x\r\n ", buf[0]);
#ifdef DEBUG
    for (i = 1; i < len-1; i++) {
      sprintf((char*)g_dbbuf, "%x ", buf[i]);
      bsp_console_sendString(g_dbbuf);
    }
    bsp_console_sendString((uint8_t*)"\r\n");
#endif
    break;
  }

}

/******************************************************************************
 *
 * Description:
 *   Process one byte of data received from the XBee module
 *
 * Params:
 *   [in] data - the received data
 *
 *****************************************************************************/
static void processByte(uint8_t data)
{
  switch(rfState) {
  case XBEE_FRAME:
    if (data == XBEE_START_DEL) {
      rfPos = 0;
      rfFrameLen = 0;
      rfState = XBEE_LENGTH;

      // start timer to make sure an entire frame is received
      // within a specific time
      rfFrameTimer = bsp_timeGet() + XBEE_RECV_FRAME_TO;
    }

    break;
  case XBEE_LENGTH:
    rfFrameLen |= (data << (8*(1-rfPos)));
    rfPos++;
    if (rfPos == 2) {
      rfPos = 0;
      rfState = XBEE_DATA;

      if (rfFrameLen > XBEE_BUF_SZ) {
        dbg("Xbee: Frame len %d > max buffer len %d\r\n", rfFrameLen, XBEE_BUF_SZ);
        rfFrameLen = XBEE_BUF_SZ;
      }

    }
    break;
  case XBEE_DATA:
    rfBuf[rfPos++] = data;
    // read up until checksum (1 byte)
    if (rfPos == rfFrameLen+1) {
      rfState = XBEE_FRAME;

      // cancel timer
      rfFrameTimer = 0;

      processFrame(rfBuf, rfPos);
    }
    break;

  }
}

/******************************************************************************
 *
 * Description:
 *   Read data from UART up until CR is found.
 *
 * Params:
 *   [in] buf - read data is copied to this buffer
 *   [in] bufLen - size of buffer
 *   [in] timeout - max number of ms to wait for data
 *
 * Returns:
 *   number of read bytes or a negative number if an error occurs
 *
 *****************************************************************************/
static int32_t readLine(uint8_t* buf, uint32_t bufLen, uint32_t timeout)
{
  uint32_t pos = 0;
  uint32_t len = 0;
  uint32_t t = 0;

  t = bsp_timeGet() + timeout;

  while(pos < bufLen && t > bsp_timeGet()) {

    len = bsp_rf_receive(&buf[pos], 1);
    if (len > 0 && buf[pos] == CR) {
      buf[pos] = '\0';
      break;
    }

    pos += len;
  }

  if (pos >= bufLen) {
    return ERROR_BUF_TOO_SMALL;
  }

  if (t <= bsp_timeGet()) {
    return ERROR_TIMEOUT;
  }

  return pos;
}

/******************************************************************************
 *
 * Description:
 *   Set the XBee module in command mode
 *
 * Returns:
 *   ERR_OK or an error code if the command failed
 *
 *****************************************************************************/
static error_t commandMode(void)
{
  uint8_t ebuf[10];
  int32_t lineLen = 0;
  error_t err = ERR_OK;

  bsp_rf_sendString((uint8_t*)"+++");

  lineLen = readLine(ebuf, 10, 1200);

  do {
    if (lineLen < 0) {
      // error while reading
      err = ERR_RF_READ_ERROR;
      break;
    }

    if (strcmp(RESP_OK, (char*)ebuf) != 0) {
      // didn't receive OK
      err = ERR_RF_CMD_ERROR;
      break;
    }
  } while(0);

  return err;
}

/******************************************************************************
 *
 * Description:
 *   AT command that expects a single line response
 *
 * Params:
 *   [in] atCmd - the AT command string
 *   [in] resp - response buffer
 *   [in] respLen - size of response buffer
 *
 * Returns:
 *   ERR_OK or an error code if the command failed
 *
 *****************************************************************************/
static error_t atGet(uint8_t* atCmd, uint8_t* resp, uint32_t respLen)
{

    int32_t lineLen = 0;
    error_t err = ERR_OK;

    bsp_rf_sendString(atCmd);
    bsp_rf_sendString((uint8_t*)XBEE_END_CMD);

    do {

      // a response is expected
      if (resp != NULL && respLen > 0) {
        lineLen = readLine(resp, respLen, 1000);

        if (lineLen < 0) {
          // error while reading
          err = ERR_RF_READ_ERROR;
          break;
        }

      }

    } while(0);

    return err;
}

/******************************************************************************
 *
 * Description:
 *   AT command that sets a value and only expects OK or ERROR as result
 *
 * Params:
 *   [in] atCmd - the AT command string
 *
 * Returns:
 *   ERR_OK or an error code if the command failed
 *
 *****************************************************************************/
static error_t atSet(uint8_t* atCmd)
{
  uint8_t b[10];
  error_t err = ERR_OK;

  err = atGet(atCmd, b, 10);
  if (err == ERR_OK) {

    if (strcmp(RESP_OK, (char*)b) != 0) {
      // didn't receive OK
      err = ERR_RF_CMD_ERROR;
    }
  }

  return err;
}

/******************************************************************************
 *
 * Description:
 *   Run sequence to reset the module
 *
 *****************************************************************************/
static void resetModule(void)
{
  // PIN12 connected to N_RST. Reset the Xbee module by
  // toggling this pin low->high
  bsp_pinDir(12, 1);

  // reset pulse must be at least 200 ns. Using 1 ms.
  bsp_pinClr(12);
  bsp_delayMs(1);
  bsp_pinSet(12);
  // wait to make sure the module has started
  bsp_delayMs(500);
}

/******************************************************************************
 * Public functions
 *****************************************************************************/


/******************************************************************************
 *
 * Description:
 *   Initialize the Xbee S6 module. This implementation will always initialize
 *   the module to API mode.
 *
 * Params:
 *   [in] type - XBEE_S6_ADHOC_CREATOR, XBEE_S6_ADHOC_JOINER or
 *               XBEE_S6_INFRASTRUCTURE
 *   [in] callbacks - callback functions where events/results are reported
 *   [in] ssid - the SSID to use. Must be the same for all devices that should
 *               be able to communicate with each other.
 *   [in] channel - only used for an Ad Hoc Creator.
 *   [in] ip - IP address of device (must be 4 bytes)
 *   [in] mask - IP address mask used by device (must be 4 bytes)
 *
 * Returns:
 *   ERR_OK or an error code if initialization failed
 *
 *****************************************************************************/
error_t xbee_s6_init(xbeeS6Type_t type, xbee_s6_callb_t* callbacks,
    uint8_t* ssid, uint8_t channel, uint8_t* ip, uint8_t* mask)
{
  error_t err = ERR_OK;
  char atBuf[20];
  bsp_comm_arg_t args;

  _cb = callbacks;

  do {

    args.arg1 = 9600;
    args.arg2 = BSP_ARG2_UART_DATABIT_8;
    args.arg3 = BSP_ARG3_UART_STOPBIT_1;
    args.arg4 = BSP_ARG4_UART_PARITY_NONE;

    bsp_rf_init(BSP_COMM_UART, &args);


    resetModule();

    // go to command mode
    err = commandMode();
    if (err != ERR_OK) {
      dbg("Xbee: +++ failed\r\n");
      break;
    }

    // set SSID
    sprintf(atBuf, "ATID%s", ssid);
    err = atSet((uint8_t*)atBuf);
    if (err != ERR_OK) {
      dbg("Xbee: ATID failed\r\n");
      break;
    }


    // static IP addresses
    err = atSet((uint8_t*)"ATMA1");
    if (err != ERR_OK) {
      dbg("Xbee: ATMA1 failed\r\n");
      break;
    }

    // IP address of device
    sprintf(atBuf, "ATMY%d.%d.%d.%d", ip[0], ip[1], ip[2], ip[3]);
    err = atSet((uint8_t*)atBuf);
    if (err != ERR_OK) {
      dbg("Xbee: ATMK failed\r\n");
      break;
    }

    // IP address mask
    sprintf(atBuf, "ATMK%d.%d.%d.%d", mask[0], mask[1], mask[2], mask[3]);
    err = atSet((uint8_t*)atBuf);
    if (err != ERR_OK) {
      dbg("Xbee: ATMK failed\r\n");
      break;
    }


    // Infrastructure => connect to an Access Point
    if (type == XBEE_S6_INFRASTRUCTURE) {

      // Network type = infrastructure
      err = atSet((uint8_t*)"ATAH2");
      if (err != ERR_OK) {
        dbg("Xbee: ATAH2 failed\r\n");
        break;
      }

      // if securityKey has been set enable security
      if (securityKey != NULL) {

        sprintf(atBuf, "ATEE%d", encProtocol);
        err = atSet((uint8_t*)atBuf);
        if (err != ERR_OK) {
          dbg("Xbee: ATEE failed\r\n");
          break;
        }

        sprintf(atBuf, "ATPK%s", securityKey);
        err = atSet((uint8_t*)atBuf);
        if (err != ERR_OK) {
          dbg("Xbee: ATPK failed\r\n");
          break;
        }


      }

    }

    // Ad Hoc mode => peer-to-peer between two nodes
    else {

      // no security (not available in Ad Hoc mode)
      err = atSet((uint8_t*)"ATEE0");
      if (err != ERR_OK) {
        dbg("Xbee: ATEE0 failed\r\n");
        break;
      }

      if (type == XBEE_S6_ADHOC_CREATOR) {

        // Ad Hoc Creator
        err = atSet((uint8_t*)"ATAH1");
        if (err != ERR_OK) {
          dbg("Xbee: ATAH1 failed\r\n");
          break;
        }

        // set channel
        sprintf(atBuf, "ATCH%d", channel);
        err = atSet((uint8_t*)atBuf);
        if (err != ERR_OK) {
          dbg("Xbee: ATCH failed\r\n");
          break;
        }

      }
      else {

        // Ad Hoc Joiner
        err = atSet((uint8_t*)"ATAH0");
        if (err != ERR_OK) {
          dbg("Xbee: ATAH0 failed\r\n");
          break;
        }

      }


    }

    // change to API mode
    err = atSet((uint8_t*)"ATAP1");
    if (err != ERR_OK) {
      dbg("Xbee: ATAP1 failed\r\n");
      break;
    }
  } while (0);


  return err;
}

/******************************************************************************
 *
 * Description:
 *   Enable encryption (only valid when network type is infrastructure).
 *
 *   Note: This function must be called before xbee_s6_init.
 *
 * Params:
 *   [in] enc - encryption protocol (WPA, WPA2, or WEP)
 *   [in] key - the security key as a null-terminated string (ASCII)
 *
 * Returns:
 *   ERR_OK or an error code if failed
 *
 *****************************************************************************/
error_t xbee_s6_enableEncryption(xbeeS6Enc_t enc, uint8_t* key)
{
  uint8_t keyLen = 0;
  if (key == NULL) {
    return ERR_ARGUMENT;
  }

  keyLen = strlen((char*)key);

  // key length must be 5 or 13 (64 or 128 bits) when using WEP)
  if (enc == XBEE_S6_ENC_WEP && keyLen != 5 && keyLen != 13) {
    return ERR_ARGUMENT;
  }

  // not accepting zero length key for WPA and WPA2 or keys > 31 characters
  if ((enc == XBEE_S6_ENC_WPA || enc == XBEE_S6_ENC_WPA2)
      && (keyLen == 0 || keyLen > 31)) {
    return ERR_ARGUMENT;
  }

  securityKey = key;
  encProtocol = enc;

  return ERR_OK;
}

/******************************************************************************
 *
 * Description:
 *   Send data to an Xbee module.
 *
 * Params:
 *   [in] addrHi - Upper 32 bits of a 64-bit node address
 *   [in] addrLo - Lower 32 bits of a 64-bit node address
 *   [in] data   - buffer containing data to send
 *   [in] len    - number of bytes to send
 *   [out] frameId - frame ID of the request
 *
 * Returns:
 *   ERR_OK or an error code if failed
 *
 *****************************************************************************/
error_t xbee_s6_send(uint8_t* destIp, uint16_t destPort, uint16_t srcPort,
    xbeeS6Protocol_t protocol, uint8_t* data, uint32_t len, uint8_t* frameId)
{
  if (!initialized) {
    return ERR_NOT_INIT;
  }

  return apiTxIPv4(destIp, destPort, srcPort, protocol, data, len, frameId);

}


/******************************************************************************
 *
 * Description:
 *   Call this function repeatedly to run the Xbee driver
 *
 *****************************************************************************/
void xbee_s6_task(void)
{
  uint32_t len = 0;
  uint8_t data = 0;

  if (rfFrameTimer > 0 && rfFrameTimer < bsp_timeGet()) {
    rfState = XBEE_FRAME;
    dbg("Xbee: Frame timer expired\r\n");
    rfFrameTimer = 0;
  }

  if (!bsp_rf_recvIsEmpty()) {

    len = bsp_rf_receive(&data, 1);
    if (len > 0) {
      processByte(data);
    }

  }
}





